@@ -23,7 +23,7 @@
  */
 
 
-#if !defined(VMX86_TOOLS) && !defined(__APPLE__)
+#if !defined(VMX86_TOOLS) && !defined(__APPLE__) && !defined(sun)
 #define FILEIO_SUPPORT_ODIRECT
 #define _GNU_SOURCE
 #endif
@@ -44,10 +44,23 @@
  * These headers are needed to get __USE_LARGEFILE, __USE_LARGEFILE64,
  * and SYS__llseek.
  */
+#   ifdef ANDROID_X86
+#      undef _GNU_SOURCE
+#   endif
 #   include <features.h>
+#   ifndef _GNU_SOURCE
+#      define _GNU_SOURCE
+#   endif
 #   include <linux/unistd.h>
+#ifdef ANDROID_X86
+#   include <sys/syscall.h>
+#else
 #   include <syscall.h>
 #endif
+#endif
+#if defined __ANDROID__
+#   include <sys/syscall.h>
+#endif
 #include <sys/stat.h>
 #include "su.h"
 
@@ -193,6 +206,7 @@ typedef struct AlignedPool {
    unsigned        numBusy;
 } AlignedPool;
 
+static Atomic_Ptr alignedPoolLockStorage;
 static AlignedPool alignedPool;
 #endif
 
@@ -811,43 +825,46 @@ FileIOCreateRetry(FileIODescriptor *file,   // OUT:
    ASSERT(file->lockToken == NULL);
    ASSERT_ON_COMPILE(FILEIO_ERROR_LAST < 16); /* See comment in fileIO.h */
 
+   FileIOResolveLockBits(&access);
+   ASSERT((access & FILEIO_OPEN_LOCKED) == 0 &&
+          (access & FILEIO_OPEN_EXCLUSIVE_LOCK) == 0);
+   /* Only ESX implements mandatory locking */
+   ASSERT((access & FILEIO_OPEN_LOCK_MANDATORY) == 0 ||
+          File_SupportsMandatoryLock(pathName));
+
 #if defined(__APPLE__)
    if (access & FILEIO_OPEN_EXCLUSIVE_LOCK_MACOS) {
       flags |= O_EXLOCK;
    }
-#elif !defined(__FreeBSD__) && !defined(sun)
-   /*
-    * If FILEIO_OPEN_EXCLUSIVE_LOCK or FILEIO_OPEN_MULTIWRITER_LOCK or
-    * (FILEIO_OPEN_ACCESS_READ | FILEIO_OPEN_LOCKED) are passed, and we are
-    * on VMFS, then pass in special flags to get exclusive, multiwriter, or
-    * cross-host read-only mode.  The first if statement is to avoid calling
-    * HostType_OSIsVMK() unless really necessary.
-    *
-    * If the above conditions are met FILEIO_OPEN_LOCKED, is filtered out --
-    * vmfs will be handling the locking, so there is no need to create
-    * lockfiles.
-    */
-   if ((access & (FILEIO_OPEN_EXCLUSIVE_LOCK |
-                  FILEIO_OPEN_MULTIWRITER_LOCK)) != 0 ||
-       (access & (FILEIO_OPEN_ACCESS_READ | FILEIO_OPEN_ACCESS_WRITE |
-                  FILEIO_OPEN_LOCKED)) ==
-       (FILEIO_OPEN_ACCESS_READ | FILEIO_OPEN_LOCKED)) {
-      if (HostType_OSIsVMK()) {
-         access &= ~FILEIO_OPEN_LOCKED;
-
-         if ((access & FILEIO_OPEN_MULTIWRITER_LOCK) != 0) {
-            flags |= O_MULTIWRITER_LOCK;
-         } else {
-            flags |= O_EXCLUSIVE_LOCK;
-         }
+#elif defined(__linux__)
+   if (((access & (FILEIO_OPEN_LOCK_MANDATORY |
+                   FILEIO_OPEN_MULTIWRITER_LOCK)) != 0) &&
+       HostType_OSIsVMK()) {
+      /* These flags are only supported on vmkernel */
+      if ((access & FILEIO_OPEN_MULTIWRITER_LOCK) != 0) {
+         flags |= O_MULTIWRITER_LOCK;
+      } else if ((access & FILEIO_OPEN_LOCK_MANDATORY) != 0) {
+         flags |= O_EXCLUSIVE_LOCK;
       }
    }
 #endif
 
+   /*
+    * Locking implementation note: this can be recursive. On ESX:
+    * FileIOCreateRetry("foo", ...ADVISORY...)
+    *  -> FileIO_Lock("foo", ...ADVISORY...)
+    *     -> FileLock_Lock("foo", ...ADVISORY...)
+    *        -> FileIOCreateRetry("foo.lck", ...MANDATORY...)
+    *           -> open("foo.lck", ...O_EXCLUSIVE_LOCK...)
+    */
+
    FileIO_Init(file, pathName);
-   ret = FileIO_Lock(file, access);
-   if (!FileIO_IsSuccess(ret)) {
-      goto error;
+   /* Mandatory file locks are only available at open() itself */
+   if ((access & FILEIO_OPEN_LOCK_ADVISORY) != 0) {
+      ret = FileIO_Lock(file, access);
+      if (!FileIO_IsSuccess(ret)) {
+         goto error;
+      }
    }
 
    if ((access & (FILEIO_OPEN_ACCESS_READ | FILEIO_OPEN_ACCESS_WRITE)) ==
@@ -1852,8 +1869,15 @@ FileIO_Pwritev(FileIODescriptor *fd,   // IN: File descriptor
             goto exit;
          }
          if (retval < leftToWrite) {
-            LOG_ONCE((LGPFX" %s wrote %"FMTSZ"d out of %"FMTSZ"u bytes.\n",
-                      __FUNCTION__, retval, leftToWrite));
+            /*
+             * Using %zd on Android generated a warning about
+             * expecting a "signed size_t" argument; casting retval to
+             * "signed size_t" generated an error, though. We've
+             * already checked for retval == -1 above, so the cast
+             * below should be OK. Refer to bug 817761.
+             */
+            LOG_ONCE((LGPFX" %s wrote %"FMTSZ"u out of %"FMTSZ"u bytes.\n",
+                      __FUNCTION__, (size_t)retval, leftToWrite));
          }
 
          buf += retval;
@@ -1877,42 +1901,16 @@ exit:
 #endif /* defined(__linux__) || defined(__APPLE__) || defined(__FreeBSD__) ||
           defined(sun) */
 
-/*
- *----------------------------------------------------------------------
- *
- * FileIO_GetSize --
- *
- *      Get size of file.
- *
- * Results:
- *      Size of file or -1.
- *
- * Side effects:
- *      None
- *
- *----------------------------------------------------------------------
- */
-
-int64
-FileIO_GetSize(const FileIODescriptor *fd)  // IN:
-{
-   struct stat statBuf;
-
-   ASSERT(fd);
-
-   return (fstat(fd->posix, &statBuf) == -1) ? -1 : statBuf.st_size;
-}
-
 
 /*
  *----------------------------------------------------------------------
  *
  * FileIO_GetAllocSize --
  *
- *      Get allocated size of file.
+ *      Get logcial and alloced size of a file.
  *
  * Results:
- *      Size of file or -1.
+ *      FileIOResult.
  *
  * Side effects:
  *      None
@@ -1920,19 +1918,32 @@ FileIO_GetSize(const FileIODescriptor *fd)  // IN:
  *----------------------------------------------------------------------
  */
 
-int64
-FileIO_GetAllocSize(const FileIODescriptor *fd)  // IN
+FileIOResult
+FileIO_GetAllocSize(const FileIODescriptor *fd,     // IN:
+                    uint64 *logicalBytes,           // OUT:
+                    uint64 *allocedBytes)           // OUT:
 {
-   struct stat s;
+   struct stat statBuf;
 
    ASSERT(fd);
 
-   /*
-    * If you don't like the magic number 512, yell at the people
-    * who wrote sys/stat.h and tell them to add a #define for it.
-    */
+   if (fstat(fd->posix, &statBuf) == -1) {
+      return FileIOErrno2Result(errno);
+   }
 
-   return (fstat(fd->posix, &s) == -1) ? -1 : s.st_blocks * 512;
+   if (logicalBytes) {
+      *logicalBytes = statBuf.st_size;
+   }
+
+   if (allocedBytes) {
+     /*
+      * If you don't like the magic number 512, yell at the people
+      * who wrote sys/stat.h and tell them to add a #define for it.
+      */
+      *allocedBytes = statBuf.st_blocks * 512;
+   }
+
+   return FILEIO_SUCCESS;
 }
 
 
@@ -1959,13 +1970,18 @@ FileIO_SetAllocSize(const FileIODescriptor *fd,  // IN:
 {
 
 #if defined(__APPLE__) || defined(__linux__)
+   FileIOResult fret;
    uint64 curSize;
    uint64 preallocLen;
 #if defined(__APPLE__)
    fstore_t prealloc;
 #endif
 
-   curSize = FileIO_GetAllocSize(fd);
+   fret = FileIO_GetAllocSize(fd, NULL, &curSize);
+   if (!FileIO_IsSuccess(fret)) {
+      return FALSE;
+   }
+
    if (curSize > size) {
       errno = EINVAL;
 
@@ -1997,12 +2013,12 @@ FileIO_SetAllocSize(const FileIODescriptor *fd,  // IN:
 /*
  *----------------------------------------------------------------------
  *
- * FileIO_GetSizeByPath --
+ * FileIO_GetAllocSizeByPath --
  *
- *      Get size of a file specified by path. 
+ *      Get the logcial and alloced size of a file specified by path.
  *
  * Results:
- *      Size of file or -1.
+ *      FileIOResult.
  *
  * Side effects:
  *      errno is set on error
@@ -2010,15 +2026,35 @@ FileIO_SetAllocSize(const FileIODescriptor *fd,  // IN:
  *----------------------------------------------------------------------
  */
 
-int64
-FileIO_GetSizeByPath(ConstUnicode pathName)  // IN:
+FileIOResult
+FileIO_GetAllocSizeByPath(ConstUnicode pathName,    // IN:
+                          uint64 *logicalBytes,     // OUT:
+                          uint64 *allocedBytes)     // OUT:
 {
-   int err;
-   struct stat statbuf;
+   struct stat statBuf;
 
-   err = Posix_Stat(pathName, &statbuf);
+   if (Posix_Stat(pathName, &statBuf) == -1) {
+      return FileIOErrno2Result(errno);
+   }
 
-   return (err == 0) ? statbuf.st_size : -1;
+   if (logicalBytes) {
+      *logicalBytes = statBuf.st_size;
+   }
+
+   if (allocedBytes) {
+#if __linux__ && defined(N_PLAT_NLM)
+      /* Netware doesn't have st_blocks.  Just fall back to GetSize. */
+      *allocedBytes = statBuf.st_size;
+#else
+     /*
+      * If you don't like the magic number 512, yell at the people
+      * who wrote sys/stat.h and tell them to add a #define for it.
+      */
+      *allocedBytes = statBuf.st_blocks * 512;
+#endif
+   }
+
+   return FILEIO_SUCCESS;
 }
 
 
@@ -2506,8 +2542,7 @@ FileIO_SupportsPrealloc(const char *pathName,  // IN:
  *
  * FileIOAligned_PoolInit --
  *
- *      Initialize alignedPool.  Must be called before FileIOAligned_PoolMalloc.
- *      This is not thread-safe and must be protected from multiple entry.
+ *      Initialize alignedPool. Must be called before FileIOAligned_PoolMalloc.
  *
  * Result:
  *      None.
@@ -2521,8 +2556,10 @@ FileIO_SupportsPrealloc(const char *pathName,  // IN:
 void
 FileIOAligned_PoolInit(void)
 {
-   ASSERT(!alignedPool.lock);
-   alignedPool.lock = MXUser_CreateExclLock("alignedPoolLock", RANK_LEAF);
+   alignedPool.lock = MXUser_CreateSingletonExclLock(&alignedPoolLockStorage,
+                                                     "alignedPoolLock",
+                                                     RANK_LEAF);
+   ASSERT_NOT_IMPLEMENTED(alignedPool.lock);
 }
 
 
