@@ -86,7 +86,9 @@ extern Bool Hostinfo_OSIsSMP(void);
 #if defined(_WIN32)
 extern Bool Hostinfo_OSIsWinNT(void);
 extern Bool Hostinfo_OSIsWow64(void);
+extern Bool Hostinfo_TSCInvariant(void);
 DWORD Hostinfo_OpenProcessBits(void);
+DWORD Hostinfo_OpenThreadBits(void);
 #else
 extern void Hostinfo_ResetProcessState(const int *keepFds,
                                        size_t numKeepFds);
