@@ -61,6 +61,7 @@
 #define VIX_USER_CREDENTIAL_NAMED_INTERACTIVE_USER    8
 #define VIX_USER_CREDENTIAL_TICKETED_SESSION          9
 #define VIX_USER_CREDENTIAL_SSPI                      10
+#define VIX_USER_CREDENTIAL_SAML_BEARER_TOKEN         11
 
 #define VIX_SHARED_SECRET_CONFIG_USER_NAME          "__VMware_Vix_Shared_Secret_1__"
 
@@ -315,6 +316,20 @@ struct VixCommandSSPI {
 VixCommandSSPI;
 
 
+/*
+ * **********************************************************
+ * This is a SAML bearer token with optional userName to specify
+ * an IdProvider store.
+ */
+typedef
+#include "vmware_pack_begin.h"
+struct VixCommandSAMLToken {
+   uint32    tokenLength;
+   uint32    nameLength;
+}
+#include "vmware_pack_end.h"
+VixCommandSAMLToken;
+
 /*
  * **********************************************************
  * Basic power op request. The response is just a generic
@@ -875,6 +890,20 @@ struct VixMsgHotRemoveDeviceRequest {
 VixMsgHotRemoveDeviceRequest;
 
 
+/*
+ * **********************************************************
+ * Change monitor type of a running VM.
+ */
+typedef
+#include "vmware_pack_begin.h"
+struct VixMsgHotChangeMonitorTypeRequest {
+   VixCommandRequestHeader header;
+   int32                   monitorType;
+}
+#include "vmware_pack_end.h"
+VixMsgHotChangeMonitorTypeRequest;
+
+
 /*
  * **********************************************************
  * Hot plug begin batch.
@@ -1079,98 +1108,6 @@ struct VixMsgRemoveBulkSnapshotRequest {
 #include "vmware_pack_end.h"
 VixMsgRemoveBulkSnapshotRequest;
 
-/*
- * Rolling Tier operations.
- */
-typedef
-#include "vmware_pack_begin.h"
-struct VixMsgAddRollingTierRequest {
-   VixCommandRequestHeader    header;
-   int32                      options;
-   uint32                     interval;
-   int                        maximum;
-   uint32                     clientFlags;
-   uint32                     propertyListLength;
-   /*
-    * Followed by:
-    *   serialized property list.
-    */
-}
-#include "vmware_pack_end.h"  
-VixMsgAddRollingTierRequest;
-   
-typedef
-#include "vmware_pack_begin.h"
-struct VixMsgAddRollingTierResponse {
-   VixCommandResponseHeader  header;
-   uint32                     tierUid;
-   uint32                     propertyListLength;
-   /*
-    * Followed by:
-    *   serialized property list.
-    */
-}
-#include "vmware_pack_end.h"  
-VixMsgAddRollingTierResponse;
-   
-typedef
-#include "vmware_pack_begin.h"
-struct VixMsgRemoveRollingTierRequest {
-   VixCommandRequestHeader    header;
-
-   int32                      options;
-   int32                      tierUid;
-   uint32                     propertyListLength;
-   /*
-    * Followed by:
-    *   serialized property list.
-    */
-}  
-#include "vmware_pack_end.h"
-VixMsgRemoveRollingTierRequest;
-   
-typedef
-#include "vmware_pack_begin.h"
-struct VixMsgRemoveRollingTierResponse {
-   VixCommandResponseHeader  header;
-   uint32                     propertyListLength;
-   /*
-    * Followed by:
-    *   serialized property list.
-    */
-}
-#include "vmware_pack_end.h"
-VixMsgRemoveRollingTierResponse;
-
-typedef
-#include "vmware_pack_begin.h"
-struct VixMsgListRollingTierRequest {
-   VixCommandRequestHeader    header;
-
-   int32                      options;
-   uint32                     propertyListLength;
-   /*
-    * Followed by:
-    *   serialized property list.
-    */
-}
-#include "vmware_pack_end.h"
-VixMsgListRollingTierRequest;
-
-typedef
-#include "vmware_pack_begin.h"
-struct VixMsgListRollingTierResponse {
-   VixCommandResponseHeader  header;
-   uint32                     propertyListLength;
-   /*
-    * Followed by:
-    *   serialized property list.
-    */
-}
-#include "vmware_pack_end.h"
-VixMsgListRollingTierResponse;
-
-
 /*
  * Stop recording or playback of a snapshot event log.
  */
@@ -1758,6 +1695,234 @@ struct VixMsgReadEnvironmentVariablesRequest {
 #include "vmware_pack_end.h"
 VixMsgReadEnvironmentVariablesRequest;
 
+/* IdProvider support */
+
+typedef
+#include "vmware_pack_begin.h"
+struct VixMsgAddAuthPrincipalRequest {
+   VixCommandRequestHeader header;
+
+   uint32                  options;
+
+   uint32                  userNameLen;
+   uint32                  pemCertLen;
+   Bool                    addMapping;
+
+   int32                   principalType;    // one of VixGuestAuthPrincipalType
+   uint32                  principalNameLen;
+   uint32                  principalCommentLen;
+
+   /* Followed by the NUL-terminated string arguments. */
+   /* char[]               userName; */
+   /* char[]               pemCert; */
+   /* char[]               principalName; */
+   /* char[]               principalComment; */
+}
+#include "vmware_pack_end.h"
+VixMsgAddAuthPrincipalRequest;
+
+typedef
+#include "vmware_pack_begin.h"
+struct VixMsgRemoveAuthPrincipalRequest {
+   VixCommandRequestHeader header;
+
+   uint32                  options;
+
+   uint32                  userNameLen;
+   uint32                  pemCertLen;
+
+   // special case for RemoveProvider:
+   // if principalType is NONE, then all principals will be removed.
+   int32                   principalType;    // one of VixGuestAuthPrincipalType
+   uint32                  principalNameLen;
+
+   /* Followed by the NUL-terminated string arguments. */
+   /* char[]               userName; */
+   /* char[]               pemCert; */
+   /* char[]               principalName; */
+}
+#include "vmware_pack_end.h"
+VixMsgRemoveAuthPrincipalRequest;
+
+typedef
+#include "vmware_pack_begin.h"
+struct VixMsgListAuthPrincipalsRequest {
+   VixCommandRequestHeader header;
+
+   uint32                  options;
+
+   uint32                  userNameLen;
+
+   /* char[]               userName; */
+}
+#include "vmware_pack_end.h"
+VixMsgListAuthPrincipalsRequest;
+
+typedef
+#include "vmware_pack_begin.h"
+struct VixMsgListMappedPrincipalsRequest {
+   VixCommandRequestHeader header;
+
+   uint32                  options;
+}
+#include "vmware_pack_end.h"
+VixMsgListMappedPrincipalsRequest;
+
+/*
+ * Windows Registry Management Support.
+ */
+typedef
+#include "vmware_pack_begin.h"
+struct VixMsgCreateRegKeyRequest {
+   VixCommandRequestHeader header;
+
+   uint32 options;
+
+   uint32 pathLength;
+   uint32 wowBitness;
+   Bool isVolatile;
+   uint32 classTypeLength;
+
+   /*
+    * Followed by NUL-terminated string arguments.
+    * char[] path;
+    * char[] classType;
+    */
+}
+#include "vmware_pack_end.h"
+VixMsgCreateRegKeyRequest;
+
+typedef
+#include "vmware_pack_begin.h"
+struct VixMsgListRegKeysRequest {
+   VixCommandRequestHeader header;
+
+   uint32 options;
+
+   /*
+    * If we need multiple roundtrips, this is the index
+    * used to identify the result being processed.
+    */
+   uint32 index;
+
+   /*
+    * If we need multiple roundtrips, this is the offset
+    * in the reply from which to send the next chunk.
+    */
+   uint32 offset;
+
+   uint32 pathLength;
+   uint32 wowBitness;
+   Bool recursive;
+   uint32 matchPatternLength;
+
+   /*
+    * Followed by NUL-terminated string arguments.
+    * char[] path;
+    * char[] matchPattern;
+    */
+}
+#include "vmware_pack_end.h"
+VixMsgListRegKeysRequest;
+
+typedef
+#include "vmware_pack_begin.h"
+struct VixMsgDeleteRegKeyRequest {
+   VixCommandRequestHeader header;
+
+   uint32 options;
+
+   uint32 pathLength;
+   uint32 wowBitness;
+   Bool recursive;
+
+   /*
+    * Followed by NUL-terminated string arguments.
+    * char[] path;
+    */
+}
+#include "vmware_pack_end.h"
+VixMsgDeleteRegKeyRequest;
+
+typedef
+#include "vmware_pack_begin.h"
+struct VixMsgSetRegValueRequest {
+   VixCommandRequestHeader header;
+
+   uint32 options;
+
+   uint32 pathLength;
+   uint32 wowBitness;
+   uint32 nameLength;
+   uint32 dataBlobType;
+   uint32 dataBlobLength;
+
+   /*
+    * Followed by NUL-terminated string arguments.
+    * char[] path;
+    * char[] name;
+    *
+    * Followed by a data blob of specified length
+    * containing information of specified type.
+    * void *dataBlob;
+    */
+}
+#include "vmware_pack_end.h"
+VixMsgSetRegValueRequest;
+
+typedef
+#include "vmware_pack_begin.h"
+struct VixMsgListRegValuesRequest {
+   VixCommandRequestHeader header;
+
+   uint32 options;
+
+   /*
+    * If we need multiple roundtrips, this is the index
+    * used to identify the result being processed.
+    */
+   uint32 index;
+
+   /*
+    * If we need multiple roundtrips, this is the offset
+    * in the reply from which to send the next chunk.
+    */
+   uint32 offset;
+
+   uint32 pathLength;
+   uint32 wowBitness;
+   Bool expandStrings;
+   uint32 matchPatternLength;
+
+   /*
+    * Followed by NUL-terminated string arguments.
+    * char[] path;
+    * char[] matchPattern;
+    */
+}
+#include "vmware_pack_end.h"
+VixMsgListRegValuesRequest;
+
+typedef
+#include "vmware_pack_begin.h"
+struct VixMsgDeleteRegValueRequest {
+   VixCommandRequestHeader header;
+
+   uint32 options;
+
+   uint32 pathLength;
+   uint32 wowBitness;
+   uint32 nameLength;
+
+   /*
+    * Followed by NUL-terminated string arguments.
+    * char[] path;
+    * char[] name;
+    */
+}
+#include "vmware_pack_end.h"
+VixMsgDeleteRegValueRequest;
+
 
 /*
  * HOWTO: Adding a new Vix Command. Step 3.
@@ -2207,13 +2372,13 @@ enum {
    VIX_COMMAND_TRANSFER_REQUEST                 = 161,
    VIX_COMMAND_TRANSFER_FINAL_DATA              = 162,
 
-   VIX_COMMAND_ADD_ROLLING_SNAPSHOT_TIER        = 163,
-   VIX_COMMAND_REMOVE_ROLLING_SNAPSHOT_TIER     = 164,
-   VIX_COMMAND_LIST_ROLLING_SNAPSHOT_TIER       = 165,
+   /* DEPRECATED VIX_COMMAND_ADD_ROLLING_SNAPSHOT_TIER        = 163,    */
+   /* DEPRECATED VIX_COMMAND_REMOVE_ROLLING_SNAPSHOT_TIER     = 164,    */
+   /* DEPRECATED VIX_COMMAND_LIST_ROLLING_SNAPSHOT_TIER       = 165,    */
 
-   VIX_COMMAND_ADD_ROLLING_SNAPSHOT_TIER_VMX    = 166,
-   VIX_COMMAND_REMOVE_ROLLING_SNAPSHOT_TIER_VMX = 167,
-   VIX_COMMAND_LIST_ROLLING_SNAPSHOT_TIER_VMX   = 168,
+   /* DEPRECATED VIX_COMMAND_ADD_ROLLING_SNAPSHOT_TIER_VMX    = 166,    */
+   /* DEPRECATED VIX_COMMAND_REMOVE_ROLLING_SNAPSHOT_TIER_VMX = 167,    */
+   /* DEPRECATED VIX_COMMAND_LIST_ROLLING_SNAPSHOT_TIER_VMX   = 168,    */
 
    VIX_COMMAND_LIST_FILESYSTEMS                 = 169,
 
@@ -2262,6 +2427,19 @@ enum {
    VIX_COMMAND_TERMINATE_PROCESS                = 193,
    VIX_COMMAND_DELETE_GUEST_FILE_EX             = 194,
    VIX_COMMAND_DELETE_GUEST_DIRECTORY_EX        = 195,
+   VIX_COMMAND_HOT_CHANGE_MONITOR_TYPE          = 196,
+
+   VIX_COMMAND_ADD_AUTH_PRINCIPAL               = 197,
+   VIX_COMMAND_REMOVE_AUTH_PRINCIPAL            = 198,
+   VIX_COMMAND_LIST_AUTH_PROVIDER_PRINCIPALS    = 199,
+   VIX_COMMAND_LIST_AUTH_MAPPED_PRINCIPALS      = 200,
+
+   VIX_COMMAND_CREATE_REGISTRY_KEY              = 201,
+   VIX_COMMAND_LIST_REGISTRY_KEYS               = 202,
+   VIX_COMMAND_DELETE_REGISTRY_KEY              = 203,
+   VIX_COMMAND_SET_REGISTRY_VALUE               = 204,
+   VIX_COMMAND_LIST_REGISTRY_VALUES             = 205,
+   VIX_COMMAND_DELETE_REGISTRY_VALUE            = 206,
 
    /*
     * HOWTO: Adding a new Vix Command. Step 2a.
@@ -2273,7 +2451,7 @@ enum {
     * Once a new command is added here, a command info field needs to be added
     * in bora/lib/foundryMsg/foundryMsg.c as well.
     */
-   VIX_COMMAND_LAST_NORMAL_COMMAND              = 196,
+   VIX_COMMAND_LAST_NORMAL_COMMAND              = 207,
 
    VIX_TEST_UNSUPPORTED_TOOLS_OPCODE_COMMAND    = 998,
    VIX_TEST_UNSUPPORTED_VMX_OPCODE_COMMAND      = 999,
@@ -2289,7 +2467,6 @@ enum {
 #define VIX_BACKDOORCOMMAND_SYNCDRIVER_FREEZE      VIX_BACKDOOR_COMMAND_VERSION"SyncDriver_Freeze"
 #define VIX_BACKDOORCOMMAND_SYNCDRIVER_THAW        VIX_BACKDOOR_COMMAND_VERSION"SyncDriver_Thaw"
 #define VIX_BACKDOORCOMMAND_GET_PROPERTIES         VIX_BACKDOOR_COMMAND_VERSION"Get_ToolsProperties"
-#define VIX_BACKDOORCOMMAND_CHECK_USER_ACCOUNT     VIX_BACKDOOR_COMMAND_VERSION"Check_User_Account"
 #define VIX_BACKDOORCOMMAND_SEND_HGFS_PACKET       VIX_BACKDOOR_COMMAND_VERSION"Send_Hgfs_Packet"
 #define VIX_BACKDOORCOMMAND_UNRECOGNIZED_COMMAND   VIX_BACKDOOR_COMMAND_VERSION"Unrecognized_Command"
 #define VIX_BACKDOORCOMMAND_COMMAND                VIX_BACKDOOR_COMMAND_VERSION"Relayed_Command"
