@@ -100,6 +100,7 @@
 #include "str.h"
 #include "util.h"
 #include "hashTable.h"
+#include "hostinfo.h"
 
 
 /*
@@ -119,8 +120,13 @@ typedef DWORD VThreadBaseKeyType;
 #define VTHREADBASE_INVALID_KEY (VThreadBaseKeyType)(TLS_OUT_OF_INDEXES)
 #else
 typedef pthread_key_t VThreadBaseKeyType;
+/* PTHREAD_KEYS_MAX not defined on Android. */
+#if defined __linux__ && !defined PTHREAD_KEYS_MAX
+#define VTHREADBASE_INVALID_KEY (VThreadBaseKeyType)(1024)
+#else
 #define VTHREADBASE_INVALID_KEY (VThreadBaseKeyType)(PTHREAD_KEYS_MAX)
 #endif
+#endif
 
 static void VThreadBaseSimpleNoID(void);
 static void VThreadBaseSimpleFreeID(void *tlsData);
@@ -205,7 +211,7 @@ static struct {
  */
 
 #if defined __GNUC__
-/* gcc-4.1.0 and gcc-4.1.1 have buggy weak-symbol optimiziation. */
+/* gcc-4.1.0 and gcc-4.1.1 have buggy weak-symbol optimization. */
 #  if __GNUC__ == 4 && __GNUC_MINOR__ == 1 && \
      (__GNUC_PATCHLEVEL__ == 0 || __GNUC_PATCHLEVEL__ == 1)
 #  error Cannot build VThreadBase with weak symbols: buggy gcc version
@@ -923,7 +929,8 @@ VThreadBaseGetNative(void)
 static Bool
 VThreadBaseNativeIsAlive(void *native)
 {
-   HANDLE hThread = OpenThread(THREAD_QUERY_INFORMATION, FALSE,
+   // Different access level due to impersonation see PR#780775
+   HANDLE hThread = OpenThread(Hostinfo_OpenThreadBits(), FALSE,
                                (DWORD)(uintptr_t)native);
 
    if (hThread != NULL) {
@@ -1007,7 +1014,7 @@ VThreadBaseSimpleNoID(void)
 
       newID = Atomic_FetchAndInc(&vthreadBaseGlobals.dynamicID);
       /*
-       * Detect VThreadID overflow (~0 is used as a sentinal).
+       * Detect VThreadID overflow (~0 is used as a sentinel).
        * Leave a space of ~10 IDs, since the increment and bounds-check
        * are not atomic.
        */
