@@ -1,5 +1,5 @@
 /*********************************************************
- * Copyright (C) 2005-2010 VMware, Inc. All rights reserved.
+ * Copyright (C) 2005-2012 VMware, Inc. All rights reserved.
  *
  * This program is free software; you can redistribute it and/or modify it
  * under the terms of the GNU General Public License as published by the
@@ -95,6 +95,17 @@ typedef enum VMCIIntrType {
  */
 #define VMCI_MAX_GUEST_QP_MEMORY (128 * 1024 * 1024)
 
+/*
+ * Queues with pre-mapped data pages must be small, so that we don't pin
+ * too much kernel memory (especially on vmkernel).  We limit a queuepair to
+ * 32 KB, or 16 KB per queue for symmetrical pairs.
+ *
+ * XXX, we are raising this limit to 4MB to support high-throughput workloads
+ * with vioi-filter.  Once we switch to rings instead of queuepairs for the
+ * page channel, we will drop this limit again.  See PR 852983.
+ */
+#define VMCI_MAX_PINNED_QP_MEMORY (4 * 1024 * 1024)
+
 /*
  * We have a fixed set of resource IDs available in the VMX.
  * This allows us to have a very simple implementation since we statically
@@ -112,6 +123,10 @@ typedef uint32 VMCI_Resource;
 #define VMCI_DOORBELL_LINK        3
 #define VMCI_DOORBELL_UNLINK      4
 #define VMCI_DOORBELL_NOTIFY      5
+/*
+ * VMCI_DATAGRAM_REQUEST_MAP and VMCI_DATAGRAM_REMOVE_MAP are
+ * obsoleted by the removal of VM to VM communication.
+ */
 #define VMCI_DATAGRAM_REQUEST_MAP 6
 #define VMCI_DATAGRAM_REMOVE_MAP  7
 #define VMCI_EVENT_SUBSCRIBE      8
@@ -124,11 +139,34 @@ typedef uint32 VMCI_Resource;
  */
 #define VMCI_HGFS_TRANSPORT       13
 #define VMCI_UNITY_PBRPC_REGISTER 14
-#define VMCI_RESOURCE_MAX         15
+/*
+ * The next two resources are for RPC calls from guest Tools, to replace the
+ * backdoor calls we used previously.  Privileged is for admin/root RPCs,
+ * unprivileged is for RPCs from any user.
+ */
+#define VMCI_RPC_PRIVILEGED       15
+#define VMCI_RPC_UNPRIVILEGED     16
+#define VMCI_RESOURCE_MAX         17
+/*
+ * The core VMCI device functionality only requires the resource IDs of
+ * VMCI_QUEUEPAIR_DETACH and below.
+ */
+#define VMCI_CORE_DEVICE_RESOURCE_MAX  VMCI_QUEUEPAIR_DETACH
+
+/*
+ * VMCI reserved host datagram resource IDs.
+ * vsock control channel has resource id 1.
+ */
+#define VMCI_DVFILTER_DATA_PATH_DATAGRAM 2
 
 /* VMCI Ids. */
 typedef uint32 VMCIId;
 
+typedef struct VMCIIdRange {
+   VMCIId begin;
+   VMCIId end;
+} VMCIIdRange;
+
 typedef struct VMCIHandle {
    VMCIId context;
    VMCIId resource;
@@ -228,14 +266,19 @@ static const VMCIHandle VMCI_INVALID_HANDLE = {VMCI_INVALID_ID,
 #define VMCI_HYPERVISOR_CONTEXT_ID 0
 
 /*
- * Well-known context id, a logical context that contains
- * a set of well-known services.
+ * Well-known context id, a logical context that contains a set of
+ * well-known services. This context ID is now obsolete.
  */
 #define VMCI_WELL_KNOWN_CONTEXT_ID 1
 
-/* Todo: Change host context id to dynamic/random id. */
+/*
+ * Context ID used by host endpoints.
+ */
 #define VMCI_HOST_CONTEXT_ID  2
 
+#define VMCI_CONTEXT_IS_VM(_cid) (VMCI_INVALID_ID != _cid && \
+                                  _cid > VMCI_HOST_CONTEXT_ID)
+
 /*
  * The VMCI_CONTEXT_RESOURCE_ID is used together with VMCI_MAKE_HANDLE to make
  * handles that refer to a specific context.
@@ -296,6 +339,8 @@ static const VMCIHandle VMCI_INVALID_HANDLE = {VMCI_INVALID_ID,
 #define VMCI_ERROR_BUSMEM_INVALIDATION   (-37)
 #define VMCI_ERROR_MODULE_NOT_LOADED     (-38)
 #define VMCI_ERROR_DEVICE_NOT_FOUND      (-39)
+#define VMCI_ERROR_QUEUEPAIR_NOT_READY   (-40)
+#define VMCI_ERROR_WOULD_BLOCK           (-41)
 
 /* VMCI clients should return error code withing this range */
 #define VMCI_ERROR_CLIENT_MIN     (-500)
@@ -314,8 +359,10 @@ typedef uint32 VMCI_Event;
 #define VMCI_EVENT_QP_RESUMED     2  // Only applicable to guest endpoints
 #define VMCI_EVENT_QP_PEER_ATTACH 3  // Applicable to guest and host
 #define VMCI_EVENT_QP_PEER_DETACH 4  // Applicable to guest and host
-#define VMCI_EVENT_MEM_ACCESS_ON  5  // Only applicable to VMX endpoints
-#define VMCI_EVENT_MEM_ACCESS_OFF 6  // Only applicable to VMX endpoints
+#define VMCI_EVENT_MEM_ACCESS_ON  5  // Applicable to VMX and vmk.  On vmk,
+                                     // this event has the Context payload type.
+#define VMCI_EVENT_MEM_ACCESS_OFF 6  // Applicable to VMX and vmk.  Same as
+                                     // above for the payload type.
 #define VMCI_EVENT_MAX            7
 
 /*
@@ -329,8 +376,12 @@ typedef uint32 VMCI_Event;
 #define VMCI_EVENT_VALID_VMX(_event) (_event == VMCI_EVENT_MEM_ACCESS_ON || \
                                       _event == VMCI_EVENT_MEM_ACCESS_OFF)
 
+#if defined(VMX86_SERVER)
+#define VMCI_EVENT_VALID(_event) (_event < VMCI_EVENT_MAX)
+#else // VMX86_SERVER
 #define VMCI_EVENT_VALID(_event) (_event < VMCI_EVENT_MAX && \
                                   !VMCI_EVENT_VALID_VMX(_event))
+#endif // VMX86_SERVER
 
 /* Reserved guest datagram resource ids. */
 #define VMCI_EVENT_HANDLER 0
@@ -796,5 +847,62 @@ VMCIQueueHeader_BufReady(const VMCIQueueHeader *consumeQHeader, // IN:
 }
 
 
-#endif
+/*
+ * Defines for the VMCI traffic filter:
+ * - VMCI_FP_<name> defines the filter protocol values
+ * - VMCI_FD_<name> defines the direction values (guest or host)
+ * - VMCI_FT_<name> are the type values (allow or deny)
+ */
 
+#define VMCI_FP_INVALID     -1
+#define VMCI_FP_HYPERVISOR   0
+#define VMCI_FP_QUEUEPAIR    (VMCI_FP_HYPERVISOR + 1)
+#define VMCI_FP_DOORBELL     (VMCI_FP_QUEUEPAIR + 1)
+#define VMCI_FP_DATAGRAM     (VMCI_FP_DOORBELL + 1)
+#define VMCI_FP_STREAMSOCK   (VMCI_FP_DATAGRAM + 1)
+#define VMCI_FP_SEQPACKET    (VMCI_FP_STREAMSOCK + 1)
+#define VMCI_FP_MAX          (VMCI_FP_SEQPACKET + 1)
+
+#define VMCI_FD_INVALID  -1
+#define VMCI_FD_GUEST     0
+#define VMCI_FD_HOST      (VMCI_FD_GUEST + 1)
+#define VMCI_FD_MAX       (VMCI_FD_HOST + 1)
+
+#define VMCI_FT_INVALID  -1
+#define VMCI_FT_ALLOW     0
+#define VMCI_FT_DENY      (VMCI_FT_ALLOW + 1)
+#define VMCI_FT_MAX       (VMCI_FT_DENY + 1)
+
+/*
+ * The filter list tracks VMCI Id ranges for a given filter.
+ */
+
+typedef struct {
+   uint32 len;
+   VMCIIdRange *list;
+} VMCIFilterList;
+
+
+/*
+ * The filter info is used to communicate the filter configuration
+ * from the VMX to the host kernel.
+ */
+
+typedef struct {
+   VA64   list;   // List of VMCIIdRange
+   uint32 len;    // Length of list
+   uint8  dir;    // VMCI_FD_X
+   uint8  proto;  // VMCI_FP_X
+   uint8  type;   // VMCI_FT_X
+} VMCIFilterInfo;
+
+/*
+ * In the host kernel, the ingoing and outgoing filters are
+ * separated. The VMCIProtoFilters type captures all filters in one
+ * direction. The VMCIFilters type captures all filters.
+ */
+
+typedef VMCIFilterList VMCIProtoFilters[VMCI_FP_MAX][VMCI_FT_MAX];
+typedef VMCIProtoFilters VMCIFilters[VMCI_FD_MAX];
+
+#endif
