@@ -1,5 +1,5 @@
 /*********************************************************
- * Copyright (C) 2006 VMware, Inc. All rights reserved.
+ * Copyright (C) 2006-2012 VMware, Inc. All rights reserved.
  *
  * This program is free software; you can redistribute it and/or modify it
  * under the terms of the GNU General Public License as published by the
@@ -34,7 +34,6 @@
 #include "vmciEvent.h"
 #include "vmciKernelAPI.h"
 #include "vmciQueuePair.h"
-#include "vmciResource.h"
 #if defined(VMKERNEL)
 #  include "vmciVmkInt.h"
 #  include "vm_libc.h"
@@ -46,8 +45,12 @@
 static void VMCIContextFreeContext(VMCIContext *context);
 static Bool VMCIContextExists(VMCIId cid);
 static int VMCIContextFireNotification(VMCIId contextID,
-                                       VMCIPrivilegeFlags privFlags,
-                                       const char *domain);
+                                       VMCIPrivilegeFlags privFlags);
+#if defined(VMKERNEL)
+static void VMCIContextReleaseGuestMemLocked(VMCIContext *context,
+                                             VMCIGuestMemID gid);
+static void VMCIContextInFilterCleanup(VMCIContext *context);
+#endif
 
 /*
  * List of current VMCI contexts.
@@ -177,36 +180,6 @@ VMCIContext_CheckAndSignalNotify(VMCIContext *context) // IN:
 #endif
 
 
-/*
- *----------------------------------------------------------------------
- *
- * VMCIContextGetDomainName --
- *
- *      Internal function for retrieving a context domain name, if
- *      supported by the platform. The returned pointer can only be
- *      assumed valid while a reference count is held on the given
- *      context.
- *
- * Results:
- *      Pointer to name if appropriate. NULL otherwise.
- *
- * Side effects:
- *      None.
- *
- *----------------------------------------------------------------------
- */
-
-static INLINE char *
-VMCIContextGetDomainName(VMCIContext *context) // IN
-{
-#ifdef VMKERNEL
-   return context->domainName;
-#else
-   return NULL;
-#endif
-}
-
-
 /*
  *----------------------------------------------------------------------
  *
@@ -231,13 +204,13 @@ VMCIContext_Init(void)
    VMCIList_Init(&contextList.head);
 
    err = VMCI_InitLock(&contextList.lock, "VMCIContextListLock",
-                       VMCI_LOCK_RANK_HIGHER);
+                       VMCI_LOCK_RANK_CONTEXTLIST);
    if (err < VMCI_SUCCESS) {
       return err;
    }
 
    err = VMCI_InitLock(&contextList.firingLock, "VMCIContextFiringLock",
-                       VMCI_LOCK_RANK_MIDDLE_LOW);
+                       VMCI_LOCK_RANK_CONTEXTFIRE);
    if (err < VMCI_SUCCESS) {
       VMCI_CleanupLock(&contextList.lock);
    }
@@ -320,12 +293,6 @@ VMCIContext_InitContext(VMCIId cid,                   // IN
 
    context->userVersion = userVersion;
 
-   context->wellKnownArray = VMCIHandleArray_Create(0);
-   if (context->wellKnownArray == NULL) {
-      result = VMCI_ERROR_NO_MEM;
-      goto error;
-   }
-
    context->queuePairArray = VMCIHandleArray_Create(0);
    if (!context->queuePairArray) {
       result = VMCI_ERROR_NO_MEM;
@@ -351,12 +318,24 @@ VMCIContext_InitContext(VMCIId cid,                   // IN
    }
 
    result = VMCI_InitLock(&context->lock, "VMCIContextLock",
-                          VMCI_LOCK_RANK_HIGHER);
+                          VMCI_LOCK_RANK_CONTEXT);
    if (result < VMCI_SUCCESS) {
       goto error;
    }
    Atomic_Write(&context->refCount, 1);
 
+#if defined(VMKERNEL)
+   result = VMCIMutex_Init(&context->guestMemMutex, "VMCIGuestMem",
+                           VMCI_SEMA_RANK_GUESTMEM);
+   if (result < VMCI_SUCCESS) {
+      VMCI_CleanupLock(&context->lock);
+      goto error;
+   }
+   context->curGuestMemID = INVALID_VMCI_GUEST_MEM_ID;
+
+   context->inFilters = NULL;
+#endif
+
    /* Inititialize host-specific VMCI context. */
    VMCIHost_InitContext(&context->hostContext, eventHnd);
 
@@ -393,10 +372,6 @@ VMCIContext_InitContext(VMCIId cid,                   // IN
    VMCI_ReleaseLock(&contextList.lock, flags);
 
 #ifdef VMKERNEL
-   /*
-    * Set default domain name.
-    */
-   VMCIContext_SetDomainName(context, "");
    VMCIContext_SetFSRState(context, FALSE, VMCI_INVALID_ID, eventHnd, FALSE);
 #endif
 
@@ -414,9 +389,6 @@ error:
    if (context->notifierArray) {
       VMCIHandleArray_Destroy(context->notifierArray);
    }
-   if (context->wellKnownArray) {
-      VMCIHandleArray_Destroy(context->wellKnownArray);
-   }
    if (context->queuePairArray) {
       VMCIHandleArray_Destroy(context->queuePairArray);
    }
@@ -492,22 +464,8 @@ VMCIContextFreeContext(VMCIContext *context)  // IN
    VMCIHandle tempHandle;
 
    /* Fire event to all contexts interested in knowing this context is dying. */
-   VMCIContextFireNotification(context->cid, context->privFlags,
-                               VMCIContextGetDomainName(context));
+   VMCIContextFireNotification(context->cid, context->privFlags);
 
-   /*
-    * Cleanup all wellknown mappings owned by context. Ideally these would
-    * be removed already but we maintain this list to make sure no resources
-    * are leaked. It is updated by the VMCIDatagramAdd/RemoveWellKnownMap.
-    */
-   ASSERT(context->wellKnownArray);
-   tempHandle = VMCIHandleArray_RemoveTail(context->wellKnownArray);
-   while (!VMCI_HANDLE_EQUAL(tempHandle, VMCI_INVALID_HANDLE)) {
-      VMCIDatagramRemoveWellKnownMap(tempHandle.resource, context->cid);
-      tempHandle = VMCIHandleArray_RemoveTail(context->wellKnownArray);
-   }
-
-#ifndef VMKERNEL
    /*
     * Cleanup all queue pair resources attached to context.  If the VM dies
     * without cleaning up, this code will make sure that no resources are
@@ -516,30 +474,18 @@ VMCIContextFreeContext(VMCIContext *context)  // IN
 
    tempHandle = VMCIHandleArray_GetEntry(context->queuePairArray, 0);
    while (!VMCI_HANDLE_EQUAL(tempHandle, VMCI_INVALID_HANDLE)) {
-      VMCIQPBroker_Lock();
-      if (VMCIQPBroker_Detach(tempHandle, context, TRUE) < VMCI_SUCCESS) {
+      if (VMCIQPBroker_Detach(tempHandle, context) < VMCI_SUCCESS) {
          /*
           * When VMCIQPBroker_Detach() succeeds it removes the handle from the
           * array.  If detach fails, we must remove the handle ourselves.
           */
          VMCIHandleArray_RemoveEntry(context->queuePairArray, tempHandle);
       }
-      VMCIQPBroker_Unlock();
       tempHandle = VMCIHandleArray_GetEntry(context->queuePairArray, 0);
    }
-#else
-   /*
-    * On ESX, all entries in the queuePairArray have been cleaned up
-    * either by the regular VMCI device destroy path or by the world
-    * cleanup destroy path. We assert that no resources are leaked.
-    */
-
-   ASSERT(VMCI_HANDLE_EQUAL(VMCIHandleArray_GetEntry(context->queuePairArray, 0),
-                            VMCI_INVALID_HANDLE));
-#endif /* !VMKERNEL */
 
    /*
-    * It is fine to destroy this without locking the callQueue, as
+    * It is fine to destroy this without locking the datagramQueue, as
     * this is the only thread having a reference to the context.
     */
 
@@ -553,11 +499,14 @@ VMCIContextFreeContext(VMCIContext *context)  // IN
    }
 
    VMCIHandleArray_Destroy(context->notifierArray);
-   VMCIHandleArray_Destroy(context->wellKnownArray);
    VMCIHandleArray_Destroy(context->queuePairArray);
    VMCIHandleArray_Destroy(context->doorbellArray);
    VMCIHandleArray_Destroy(context->pendingDoorbellArray);
    VMCI_CleanupLock(&context->lock);
+#if defined(VMKERNEL)
+   VMCIContextInFilterCleanup(context);
+   VMCIMutex_Destroy(&context->guestMemMutex);
+#endif
    VMCIHost_ReleaseContext(&context->hostContext);
 #ifndef VMX86_SERVER
 #  ifdef __linux__
@@ -661,6 +610,18 @@ VMCIContext_EnqueueDatagram(VMCIId cid,        // IN: Target VM
    VMCIList_InitEntry(&dqEntry->listItem);
 
    VMCI_GrabLock(&context->lock, &flags);
+
+#if defined(VMKERNEL)
+   if (context->inFilters != NULL) {
+      if (VMCIFilterDenyDgIn(context->inFilters->filters, dg)) {
+         VMCI_ReleaseLock(&context->lock, flags);
+         VMCIContext_Release(context);
+         VMCI_FreeKernelMem(dqEntry, sizeof *dqEntry);
+         return VMCI_ERROR_NO_ACCESS;
+      }
+   }
+#endif
+
    /*
     * We put a higher limit on datagrams from the hypervisor.  If the pending
     * datagram is not from hypervisor, then we check if enqueueing it would
@@ -1051,6 +1012,7 @@ VMCIContext_FindAndUpdateSrcFSR(VMCIId migrateCid,      // IN
  *
  *----------------------------------------------------------------------
  */
+
 Bool
 VMCIContext_IsActiveHnd(VMCIContext *context, // IN
                         uintptr_t eventHnd)   // IN
@@ -1066,6 +1028,36 @@ VMCIContext_IsActiveHnd(VMCIContext *context, // IN
 }
 
 
+/*
+ *----------------------------------------------------------------------
+ *
+ * VMCIContext_GetActiveHnd --
+ *
+ *      Returns the curent event handle.
+ *
+ * Results:
+ *      The current active handle.
+ *
+ * Side effects:
+ *      None.
+ *
+ *----------------------------------------------------------------------
+ */
+
+uintptr_t
+VMCIContext_GetActiveHnd(VMCIContext *context) // IN
+{
+   VMCILockFlags flags;
+   uintptr_t activeHnd;
+
+   ASSERT(context);
+   VMCI_GrabLock(&context->lock, &flags);
+   activeHnd = VMCIHost_GetActiveHnd(&context->hostContext);
+   VMCI_ReleaseLock(&context->lock, flags);
+   return activeHnd;
+}
+
+
 /*
  *----------------------------------------------------------------------
  *
@@ -1297,81 +1289,6 @@ VMCIContext_GetPrivFlags(VMCIId contextID)  // IN
 }
 
 
-/*
- *----------------------------------------------------------------------
- *
- * VMCIContext_AddWellKnown --
- *
- *      Wrapper to call VMCIHandleArray_AppendEntry().
- *
- * Results:
- *      VMCI_SUCCESS on success, error code otherwise.
- *
- * Side effects:
- *      As in VMCIHandleArray_AppendEntry().
- *
- *----------------------------------------------------------------------
- */
-
-int
-VMCIContext_AddWellKnown(VMCIId contextID,    // IN:
-                         VMCIId wellKnownID)  // IN:
-{
-   VMCILockFlags flags;
-   VMCIHandle wkHandle;
-   VMCIContext *context = VMCIContext_Get(contextID);
-   if (context == NULL) {
-      return VMCI_ERROR_NOT_FOUND;
-   }
-   wkHandle = VMCI_MAKE_HANDLE(VMCI_WELL_KNOWN_CONTEXT_ID, wellKnownID);
-   VMCI_GrabLock(&context->lock, &flags);
-   VMCIHandleArray_AppendEntry(&context->wellKnownArray, wkHandle);
-   VMCI_ReleaseLock(&context->lock, flags);
-   VMCIContext_Release(context);
-
-   return VMCI_SUCCESS;
-}
-
-
-/*
- *----------------------------------------------------------------------
- *
- * VMCIContext_RemoveWellKnown --
- *
- *      Wrapper to call VMCIHandleArray_RemoveEntry().
- *
- * Results:
- *      VMCI_SUCCESS if removed, error code otherwise.
- *
- * Side effects:
- *      As in VMCIHandleArray_RemoveEntry().
- *
- *----------------------------------------------------------------------
- */
-
-int
-VMCIContext_RemoveWellKnown(VMCIId contextID,    // IN:
-                            VMCIId wellKnownID)  // IN:
-{
-   VMCILockFlags flags;
-   VMCIHandle wkHandle, tmpHandle;
-   VMCIContext *context = VMCIContext_Get(contextID);
-   if (context == NULL) {
-      return VMCI_ERROR_NOT_FOUND;
-   }
-   wkHandle = VMCI_MAKE_HANDLE(VMCI_WELL_KNOWN_CONTEXT_ID, wellKnownID);
-   VMCI_GrabLock(&context->lock, &flags);
-   tmpHandle = VMCIHandleArray_RemoveEntry(context->wellKnownArray, wkHandle);
-   VMCI_ReleaseLock(&context->lock, flags);
-   VMCIContext_Release(context);
-
-   if (VMCI_HANDLE_EQUAL(tmpHandle, VMCI_INVALID_HANDLE)) {
-      return VMCI_ERROR_NOT_FOUND;
-   }
-   return VMCI_SUCCESS;
-}
-
-
 /*
  *----------------------------------------------------------------------
  *
@@ -1402,6 +1319,14 @@ VMCIContext_AddNotification(VMCIId contextID,  // IN:
       return VMCI_ERROR_NOT_FOUND;
    }
 
+   if (VMCI_CONTEXT_IS_VM(contextID) && VMCI_CONTEXT_IS_VM(remoteCID)) {
+      VMCI_DEBUG_LOG(4, (LGPFX"Context removed notifications for other VMs not "
+                         "supported (src=0x%x, remote=0x%x).\n",
+                         contextID, remoteCID));
+      result = VMCI_ERROR_DST_UNREACHABLE;
+      goto out;
+   }
+
    if (context->privFlags & VMCI_PRIVILEGE_FLAG_RESTRICTED) {
       result = VMCI_ERROR_NO_ACCESS;
       goto out;
@@ -1485,8 +1410,7 @@ VMCIContext_RemoveNotification(VMCIId contextID,  // IN:
 
 static int
 VMCIContextFireNotification(VMCIId contextID,             // IN
-                            VMCIPrivilegeFlags privFlags, // IN
-                            const char *domain)           // IN
+                            VMCIPrivilegeFlags privFlags) // IN
 {
    uint32 i, arraySize;
    VMCIListItem *next;
@@ -1521,8 +1445,7 @@ VMCIContextFireNotification(VMCIId contextID,             // IN
        */
 
       if (VMCIHandleArray_HasEntry(subCtx->notifierArray, contextHandle) &&
-          !VMCIDenyInteraction(privFlags, subCtx->privFlags, domain,
-                               VMCIContextGetDomainName(subCtx))) {
+          !VMCIDenyInteraction(privFlags, subCtx->privFlags)) {
          VMCIHandleArray_AppendEntry(&subscriberArray,
                                      VMCI_MAKE_HANDLE(subCtx->cid,
                                                       VMCI_EVENT_HANDLER));
@@ -1610,9 +1533,15 @@ VMCIContext_GetCheckpointState(VMCIId contextID,    // IN:
       array = context->notifierArray;
       getContextID = TRUE;
    } else if (cptType == VMCI_WELLKNOWN_CPT_STATE) {
-      ASSERT(context->wellKnownArray);
-      array = context->wellKnownArray;
-      getContextID = FALSE;
+      /*
+       * For compatibility with VMX'en with VM to VM communication, we
+       * always return zero wellknown handles.
+       */
+
+      *bufSize = 0;
+      *cptBufPtr = NULL;
+      result = VMCI_SUCCESS;
+      goto release;
    } else if (cptType == VMCI_DOORBELL_CPT_STATE) {
       ASSERT(context->doorbellArray);
       array = context->doorbellArray;
@@ -1696,19 +1625,27 @@ VMCIContext_SetCheckpointState(VMCIId contextID, // IN:
    uint32 numIDs = bufSize / sizeof(VMCIId);
    ASSERT(cptBuf);
 
-   if (cptType != VMCI_NOTIFICATION_CPT_STATE &&
-       cptType != VMCI_WELLKNOWN_CPT_STATE) {
+   if (cptType == VMCI_WELLKNOWN_CPT_STATE && numIDs > 0) {
+      /*
+       * We would end up here if VMX with VM to VM communication
+       * attempts to restore a checkpoint with wellknown handles.
+       */
+
+      VMCI_WARNING((LGPFX"Attempt to restore checkpoint with obsolete "
+                    "wellknown handles.\n"));
+      return VMCI_ERROR_OBSOLETE;
+   }
+
+   if (cptType != VMCI_NOTIFICATION_CPT_STATE) {
       VMCI_DEBUG_LOG(4, (LGPFX"Invalid cpt state (type=%d).\n", cptType));
       return VMCI_ERROR_INVALID_ARGS;
    }
 
    for (i = 0; i < numIDs && result == VMCI_SUCCESS; i++) {
       currentID = ((VMCIId *)cptBuf)[i];
-      if (cptType == VMCI_NOTIFICATION_CPT_STATE) {
-         result = VMCIContext_AddNotification(contextID, currentID);
-      } else if (cptType == VMCI_WELLKNOWN_CPT_STATE) {
-         result = VMCIDatagramRequestWellKnownMap(currentID, contextID,
-                                                  VMCIContext_GetPrivFlags(contextID));
+      result = VMCIContext_AddNotification(contextID, currentID);
+      if (result != VMCI_SUCCESS) {
+         break;
       }
    }
    if (result != VMCI_SUCCESS) {
@@ -2034,23 +1971,20 @@ VMCIContext_NotifyDoorbell(VMCIId srcCID,                   // IN
    dstContext = VMCIContext_Get(handle.context);
    if (dstContext == NULL) {
       VMCI_DEBUG_LOG(4, (LGPFX"Invalid context (ID=0x%x).\n", handle.context));
-      return VMCI_ERROR_INVALID_ARGS;
+      return VMCI_ERROR_NOT_FOUND;
    }
 
    if (srcCID != handle.context) {
       VMCIPrivilegeFlags dstPrivFlags;
-#if !defined(VMKERNEL)
-      char *srcDomain = NULL;
-#else
-      char srcDomain[VMCI_DOMAIN_NAME_MAXLEN];
 
-      result = VMCIContext_GetDomainName(srcCID, srcDomain, sizeof srcDomain);
-      if (result < VMCI_SUCCESS) {
-         VMCI_WARNING((LGPFX"Failed to get domain name for source context "
-                       "(ID=0x%x).\n", srcCID));
+      if (VMCI_CONTEXT_IS_VM(srcCID) && VMCI_CONTEXT_IS_VM(handle.context)) {
+         VMCI_DEBUG_LOG(4, (LGPFX"Doorbell notification from VM to VM not "
+                            "supported (src=0x%x, dst=0x%x).\n",
+                            srcCID, handle.context));
+         result = VMCI_ERROR_DST_UNREACHABLE;
          goto out;
       }
-#endif
+
       result = VMCIDoorbellGetPrivFlags(handle, &dstPrivFlags);
       if (result < VMCI_SUCCESS) {
          VMCI_WARNING((LGPFX"Failed to get privilege flags for destination "
@@ -2064,8 +1998,7 @@ VMCIContext_NotifyDoorbell(VMCIId srcCID,                   // IN
          srcPrivFlags = VMCIContext_GetPrivFlags(srcCID);
       }
 
-      if (VMCIDenyInteraction(srcPrivFlags, dstPrivFlags, srcDomain,
-                              VMCIContextGetDomainName(dstContext))) {
+      if (VMCIDenyInteraction(srcPrivFlags, dstPrivFlags)) {
          result = VMCI_ERROR_NO_ACCESS;
          goto out;
       }
@@ -2076,6 +2009,13 @@ VMCIContext_NotifyDoorbell(VMCIId srcCID,                   // IN
    } else {
       VMCI_GrabLock(&dstContext->lock, &flags);
 
+#if defined(VMKERNEL)
+      if (dstContext->inFilters != NULL &&
+          VMCIFilterProtoDeny(dstContext->inFilters->filters, handle.resource,
+                              VMCI_FP_DOORBELL)) {
+         result = VMCI_ERROR_NO_ACCESS;
+      } else
+#endif // VMKERNEL
       if (!VMCIHandleArray_HasEntry(dstContext->doorbellArray, handle)) {
          result = VMCI_ERROR_NOT_FOUND;
       } else {
@@ -2151,12 +2091,15 @@ VMCIContext_SignalPendingDoorbells(VMCIId contextID)
 /*
  *----------------------------------------------------------------------
  *
- * VMCIContext_SetDomainName --
+ * VMCIContext_SignalPendingDatagrams --
  *
- *      Sets the domain name of the given context.
+ *      Signals the guest if any datagrams are pending. This is used
+ *      after the VMCI device is unquiesced to ensure that no pending
+ *      datagrams go unnoticed, since signals may not be fully
+ *      processed while the device is quiesced.
  *
  * Results:
- *      VMCI_SUCCESS on success, error code otherwise.
+ *      None.
  *
  * Side effects:
  *      None.
@@ -2164,75 +2107,30 @@ VMCIContext_SignalPendingDoorbells(VMCIId contextID)
  *----------------------------------------------------------------------
  */
 
-int
-VMCIContext_SetDomainName(VMCIContext *context,   // IN;
-                          const char *domainName) // IN:
-{
-   size_t domainNameLen;
-
-   if (!context || !domainName) {
-      return VMCI_ERROR_INVALID_ARGS;
-   }
-
-   domainNameLen = strlen(domainName);
-   if (domainNameLen >= sizeof context->domainName) {
-      return VMCI_ERROR_NO_MEM;
-   }
-
-   memcpy(context->domainName, domainName, domainNameLen + 1);
-
-   return VMCI_SUCCESS;
-}
-
-
-/*
- *----------------------------------------------------------------------
- *
- * VMCIContext_GetDomainName --
- *
- *      Returns the domain name of the given context.
- *
- * Results:
- *      VMCI_SUCCESS on success, error code otherwise.
- *
- * Side effects:
- *      None.
- *
- *----------------------------------------------------------------------
- */
-
-int
-VMCIContext_GetDomainName(VMCIId contextID,         // IN:
-                          char *domainName,         // OUT:
-                          size_t domainNameBufSize) // IN:
+void
+VMCIContext_SignalPendingDatagrams(VMCIId contextID)
 {
+   Bool pending;
    VMCIContext *context;
-   int rv = VMCI_SUCCESS;
-   size_t domainNameLen;
-
-   if (contextID == VMCI_INVALID_ID || !domainName || !domainNameBufSize) {
-      return VMCI_ERROR_INVALID_ARGS;
-   }
+   VMCILockFlags flags;
 
    context = VMCIContext_Get(contextID);
    if (!context) {
-      return VMCI_ERROR_NOT_FOUND;
+      ASSERT(FALSE);
+      return;
    }
 
-   domainNameLen = strlen(context->domainName);
-   if (domainNameLen >= domainNameBufSize) {
-      rv = VMCI_ERROR_NO_MEM;
-      goto out;
+   VMCI_GrabLock(&context->lock, &flags);
+   pending = context->pendingDatagrams;
+   VMCI_ReleaseLock(&context->lock, flags);
+
+   if (pending) {
+      VMCIHost_SignalCall(&context->hostContext);
    }
 
-   memcpy(domainName, context->domainName, domainNameLen + 1);
-
-out:
    VMCIContext_Release(context);
-   return rv;
 }
 
-
 #endif // defined(VMKERNEL)
 
 
@@ -2350,7 +2248,7 @@ VMCI_IsContextOwner(VMCIId contextID,   // IN
  *      by this caller.
  *
  * Results:
- *      VMCI_SUCCESS on success, error code otherwise.
+ *      TRUE if context supports host queue pairs, FALSE otherwise.
  *
  * Side effects:
  *      None.
@@ -2370,3 +2268,373 @@ VMCIContext_SupportsHostQP(VMCIContext *context)    // IN: Context structure
    return TRUE;
 #endif
 }
+
+
+
+
+/*
+ *----------------------------------------------------------------------
+ *
+ * VMCIContext_QueuePairCreate --
+ *
+ *      Registers that a new queue pair handle has been allocated by
+ *      the context.
+ *
+ * Results:
+ *      VMCI_SUCCESS on success, appropriate error code otherewise.
+ *
+ * Side effects:
+ *      None.
+ *
+ *----------------------------------------------------------------------
+ */
+
+int
+VMCIContext_QueuePairCreate(VMCIContext *context, // IN: Context structure
+                            VMCIHandle handle)    // IN
+{
+   int result;
+
+   if (context == NULL || VMCI_HANDLE_INVALID(handle)) {
+      return VMCI_ERROR_INVALID_ARGS;
+   }
+
+   if (!VMCIHandleArray_HasEntry(context->queuePairArray, handle)) {
+      VMCIHandleArray_AppendEntry(&context->queuePairArray, handle);
+      result = VMCI_SUCCESS;
+   } else {
+      result = VMCI_ERROR_DUPLICATE_ENTRY;
+   }
+
+   return result;
+}
+
+
+/*
+ *----------------------------------------------------------------------
+ *
+ * VMCIContext_QueuePairDestroy --
+ *
+ *      Unregisters a queue pair handle that was previously registered
+ *      with VMCIContext_QueuePairCreate.
+ *
+ * Results:
+ *      VMCI_SUCCESS on success, appropriate error code otherewise.
+ *
+ * Side effects:
+ *      None.
+ *
+ *----------------------------------------------------------------------
+ */
+
+int
+VMCIContext_QueuePairDestroy(VMCIContext *context, // IN: Context structure
+                             VMCIHandle handle)    // IN
+{
+   VMCIHandle removedHandle;
+
+   if (context == NULL || VMCI_HANDLE_INVALID(handle)) {
+      return VMCI_ERROR_INVALID_ARGS;
+   }
+
+   removedHandle = VMCIHandleArray_RemoveEntry(context->queuePairArray, handle);
+
+   if (VMCI_HANDLE_INVALID(removedHandle)) {
+      return VMCI_ERROR_NOT_FOUND;
+   } else {
+      return VMCI_SUCCESS;
+   }
+}
+
+
+/*
+ *----------------------------------------------------------------------
+ *
+ * VMCIContext_QueuePairExists --
+ *
+ *      Determines whether a given queue pair handle is registered
+ *      with the given context.
+ *
+ * Results:
+ *      TRUE, if queue pair is registered with context. FALSE, otherwise.
+ *
+ * Side effects:
+ *      None.
+ *
+ *----------------------------------------------------------------------
+ */
+
+Bool
+VMCIContext_QueuePairExists(VMCIContext *context, // IN: Context structure
+                            VMCIHandle handle)    // IN
+{
+   Bool result;
+
+   if (context == NULL || VMCI_HANDLE_INVALID(handle)) {
+      return VMCI_ERROR_INVALID_ARGS;
+   }
+
+   result = VMCIHandleArray_HasEntry(context->queuePairArray, handle);
+
+   return result;
+}
+
+
+/*
+ *----------------------------------------------------------------------
+ *
+ * VMCIContext_RegisterGuestMem --
+ *
+ *      Tells the context that guest memory is available for
+ *      access. This should only be used when unquiescing the VMCI
+ *      device of a guest.
+ *
+ * Results:
+ *      None.
+ *
+ * Side effects:
+ *      Notifies host side endpoints of queue pairs that the queue pairs
+ *      can be accessed.
+ *
+ *----------------------------------------------------------------------
+ */
+
+void
+VMCIContext_RegisterGuestMem(VMCIContext *context, // IN: Context structure
+                             VMCIGuestMemID gid)   // IN: Reference to guest
+{
+#ifdef VMKERNEL
+   uint32 numQueuePairs;
+   uint32 cur;
+
+   VMCIMutex_Acquire(&context->guestMemMutex);
+
+   if (context->curGuestMemID != INVALID_VMCI_GUEST_MEM_ID) {
+      if (context->curGuestMemID != gid) {
+         /*
+          * The guest memory has been registered with a different guest
+          * memory ID. This is possible if we attempt to continue the
+          * execution of the source VMX following a failed FSR.
+          */
+
+         VMCIContextReleaseGuestMemLocked(context, context->curGuestMemID);
+      } else {
+         /*
+          * When unquiescing the device during a restore sync not part
+          * of an FSR, we will already have registered the guest
+          * memory when creating the device, so we don't need to do it
+          * again. Also, there are no active queue pairs at this
+          * point, so nothing to do.
+          */
+
+         ASSERT(VMCIHandleArray_GetSize(context->queuePairArray) == 0);
+         goto out;
+      }
+   }
+   context->curGuestMemID = gid;
+
+   /*
+    * It is safe to access the queue pair array here, since no changes
+    * to the queuePairArray can take place until after the unquiescing
+    * is complete.
+    */
+
+   numQueuePairs = VMCIHandleArray_GetSize(context->queuePairArray);
+   for (cur = 0; cur < numQueuePairs; cur++) {
+      VMCIHandle handle;
+      handle = VMCIHandleArray_GetEntry(context->queuePairArray, cur);
+      if (!VMCI_HANDLE_EQUAL(handle, VMCI_INVALID_HANDLE)) {
+         int res;
+
+         res = VMCIQPBroker_Map(handle, context, NULL);
+         if (res < VMCI_SUCCESS) {
+            VMCI_WARNING(("Failed to map guest memory for queue pair "
+                          "(handle=0x%x:0x%x, res=%d).\n",
+                          handle.context, handle.resource, res));
+         }
+      }
+   }
+
+out:
+   VMCIMutex_Release(&context->guestMemMutex);
+#endif
+}
+
+
+#ifdef VMKERNEL
+/*
+ *----------------------------------------------------------------------
+ *
+ * VMCIContextReleaseGuestMemLocked --
+ *
+ *      A version of VMCIContext_ReleaseGuestMem that assumes that the
+ *      guest mem lock is already held.
+ *
+ * Results:
+ *      None.
+ *
+ * Side effects:
+ *      None.
+ *
+ *----------------------------------------------------------------------
+ */
+
+static void
+VMCIContextReleaseGuestMemLocked(VMCIContext *context, // IN: Context structure
+                                 VMCIGuestMemID gid)   // IN: Reference to guest
+{
+   uint32 numQueuePairs;
+   uint32 cur;
+
+   /*
+    * It is safe to access the queue pair array here, since no changes
+    * to the queuePairArray can take place when the the quiescing
+    * has been initiated.
+    */
+
+   numQueuePairs = VMCIHandleArray_GetSize(context->queuePairArray);
+   for (cur = 0; cur < numQueuePairs; cur++) {
+      VMCIHandle handle;
+      handle = VMCIHandleArray_GetEntry(context->queuePairArray, cur);
+      if (!VMCI_HANDLE_EQUAL(handle, VMCI_INVALID_HANDLE)) {
+         int res;
+
+         res = VMCIQPBroker_Unmap(handle, context, gid);
+         if (res < VMCI_SUCCESS) {
+            VMCI_WARNING(("Failed to unmap guest memory for queue pair "
+                          "(handle=0x%x:0x%x, res=%d).\n",
+                          handle.context, handle.resource, res));
+         }
+      }
+   }
+}
+#endif
+
+
+/*
+ *----------------------------------------------------------------------
+ *
+ * VMCIContext_ReleaseGuestMem --
+ *
+ *      Releases all the contexts references to guest memory, if the
+ *      caller identified by the gid was the last one to register the
+ *      guest memory. This should only be used when quiescing or
+ *      cleaning up the VMCI device of a guest.
+ *
+ * Results:
+ *      None.
+ *
+ * Side effects:
+ *      None.
+ *
+ *----------------------------------------------------------------------
+ */
+
+void
+VMCIContext_ReleaseGuestMem(VMCIContext *context, // IN: Context structure
+                            VMCIGuestMemID gid)   // IN: Reference to guest
+{
+#ifdef VMKERNEL
+   VMCIMutex_Acquire(&context->guestMemMutex);
+
+   if (context->curGuestMemID == gid) {
+      /*
+       * In the case of an FSR, we may have multiple VMX'en
+       * registering and releasing guest memory concurrently. The
+       * common case is that the source will clean up its device state
+       * after a successful FSR, where the destination may already
+       * have registered guest memory. So we only release guest
+       * memory, if this is the same gid, that registered the memory.
+       */
+
+      VMCIContextReleaseGuestMemLocked(context, gid);
+      context->curGuestMemID = INVALID_VMCI_GUEST_MEM_ID;
+   }
+
+   VMCIMutex_Release(&context->guestMemMutex);
+#endif
+}
+
+#if defined(VMKERNEL)
+
+/*
+ *----------------------------------------------------------------------
+ *
+ * VMCIContext_FilterSet --
+ *
+ *      Sets an ingoing (host to guest) filter for the VMCI firewall of the
+ *      given context. If a filter list already exists for the given filter
+ *      entry, the old entry will be deleted. It is assumed that the list
+ *      can be used as is, and that the memory backing it will be freed by the
+ *      VMCI Context module once the filter is deleted.
+ *
+ * Results:
+ *      VMCI_SUCCESS on success,
+ *      VMCI_ERROR_NOT_FOUND if there is no active context linked to the cid,
+ *      VMCI_ERROR_INVALID_ARGS if a non-VM cid is specified.
+ *
+ * Side effects:
+ *      None.
+ *
+ *----------------------------------------------------------------------
+ */
+
+int
+VMCIContext_FilterSet(VMCIId cid,                // IN
+                      VMCIFilterState *filters)  // IN
+{
+   VMCIContext *context;
+   VMCILockFlags flags;
+   VMCIFilterState *oldState;
+
+   if (!VMCI_CONTEXT_IS_VM(cid)) {
+      return VMCI_ERROR_INVALID_ARGS;
+   }
+
+   context = VMCIContext_Get(cid);
+   if (!context) {
+      return VMCI_ERROR_NOT_FOUND;
+   }
+
+   VMCI_GrabLock(&context->lock, &flags);
+
+   oldState = context->inFilters;
+   context->inFilters = filters;
+
+   VMCI_ReleaseLock(&context->lock, flags);
+   if (oldState) {
+      VMCIVMKDevFreeFilterState(oldState);
+   }
+   VMCIContext_Release(context);
+
+   return VMCI_SUCCESS;
+}
+
+
+/*
+ *----------------------------------------------------------------------
+ *
+ * VMCIContextInFilterCleanup --
+ *
+ *      When a context is destroyed, all filters will be deleted.
+ *
+ * Results:
+ *      None.
+ *
+ * Side effects:
+ *      None.
+ *
+ *----------------------------------------------------------------------
+ */
+
+static void
+VMCIContextInFilterCleanup(VMCIContext *context)
+{
+   if (context->inFilters != NULL) {
+      VMCIVMKDevFreeFilterState(context->inFilters);
+      context->inFilters = NULL;
+   }
+}
+
+#endif
+
