@@ -68,6 +68,13 @@
 
 struct VMCIQueueKernelIf {
    struct page **page;
+   struct page **headerPage;
+   void *va;
+   VMCIMutex __mutex;
+   VMCIMutex *mutex;
+   Bool host;
+   Bool isDataMapped;
+   size_t numPages;
 };
 
 typedef struct VMCIDelayedWorkInfo {
@@ -85,7 +92,7 @@ typedef struct VMCIDelayedWorkInfo {
  *      Initializes the lock. Must be called before use.
  *
  * Results:
- *      None
+ *      Always VMCI_SUCCESS.
  *
  * Side effects:
  *      Thread can block.
@@ -93,12 +100,13 @@ typedef struct VMCIDelayedWorkInfo {
  *-----------------------------------------------------------------------------
  */
 
-void
+int
 VMCI_InitLock(VMCILock *lock,    // IN:
               char *name,        // IN: Unused on Linux
               VMCILockRank rank) // IN: Unused on Linux
 {
    spin_lock_init(lock);
+   return VMCI_SUCCESS;
 }
 
 
@@ -705,8 +713,8 @@ VMCI_SignalEvent(VMCIEvent *event)  // IN:
 
 void
 VMCI_WaitOnEvent(VMCIEvent *event,              // IN:
-		 VMCIEventReleaseCB releaseCB,  // IN:
-		 void *clientData)              // IN:
+                 VMCIEventReleaseCB releaseCB,  // IN:
+                 void *clientData)              // IN:
 {
    /*
     * XXX Should this be a TASK_UNINTERRUPTIBLE wait? I'm leaving it
@@ -778,7 +786,9 @@ VMCI_WaitOnEventInterruptible(VMCIEvent *event,              // IN:
  */
 
 int
-VMCIMutex_Init(VMCIMutex *mutex) // IN:
+VMCIMutex_Init(VMCIMutex *mutex,  // IN
+               char *name,        // IN: Unused
+               VMCILockRank rank) // IN: Unused
 {
    sema_init(mutex, 1);
    return VMCI_SUCCESS;
@@ -877,7 +887,8 @@ VMCIMutex_Release(VMCIMutex *mutex) // IN:
  */
 
 void *
-VMCI_AllocQueue(uint64 size) // IN: size of queue (not including header)
+VMCI_AllocQueue(uint64 size,  // IN: size of queue (not including header)
+                uint32 flags) // IN: queuepair flags
 {
    uint64 i;
    VMCIQueue *queue;
@@ -888,6 +899,27 @@ VMCI_AllocQueue(uint64 size) // IN: size of queue (not including header)
       sizeof *queue + sizeof *(queue->kernelIf) +
       numDataPages * sizeof *(queue->kernelIf->page);
 
+   /*
+    * Size should be enforced by VMCIQPair_Alloc(), double-check here.
+    * Allocating too much on Linux can cause the system to become
+    * unresponsive, because we allocate page-by-page, and we allow the
+    * system to wait for pages rather than fail.
+    */
+
+   if (size > VMCI_MAX_GUEST_QP_MEMORY) {
+      ASSERT(FALSE);
+      return NULL;
+   }
+
+   /*
+    * If pinning is requested then double-check the size of the queue.
+    * VMCIQPair_Alloc() will do this for the total queuepair size.
+    */
+
+   if ((flags & VMCI_QPFLAG_PINNED) && size > VMCI_MAX_PINNED_QP_MEMORY) {
+      return NULL;
+   }
+
    qHeader = (VMCIQueueHeader *)vmalloc(queueSize);
    if (!qHeader) {
       return NULL;
@@ -895,21 +927,40 @@ VMCI_AllocQueue(uint64 size) // IN: size of queue (not including header)
 
    queue = (VMCIQueue *)((uint8 *)qHeader + PAGE_SIZE);
    queue->qHeader = qHeader;
+   queue->savedHeader = NULL;
    queue->kernelIf = (VMCIQueueKernelIf *)((uint8 *)queue + sizeof *queue);
+   queue->kernelIf->headerPage = NULL; // Unused in guest.
    queue->kernelIf->page = (struct page **)((uint8 *)queue->kernelIf +
                                             sizeof *(queue->kernelIf));
+   queue->kernelIf->va = NULL;
+   queue->kernelIf->host = FALSE;
+   queue->kernelIf->isDataMapped = FALSE;
 
    for (i = 0; i < numDataPages; i++) {
       queue->kernelIf->page[i] = alloc_pages(GFP_KERNEL, 0);
       if (!queue->kernelIf->page[i]) {
-         while (i) {
-            __free_page(queue->kernelIf->page[--i]);
-         }
-         vfree(qHeader);
+         VMCI_FreeQueue(queue, i * PAGE_SIZE);
          return NULL;
       }
    }
 
+   /*
+    * alloc_pages() returns pinned PAs, but we need a permanent mapping to VA
+    * if the caller has requested pinned queuepairs.  Map all of them into
+    * kernel VA now, for the lifetime of the queue.  The page VAs will be
+    * contiguous.
+    */
+
+   if (flags & VMCI_QPFLAG_PINNED) {
+      queue->kernelIf->va = vmap(queue->kernelIf->page, numDataPages, VM_MAP,
+                                 PAGE_KERNEL);
+      if (NULL == queue->kernelIf->va) {
+         VMCI_FreeQueue(queue, numDataPages * PAGE_SIZE);
+         return NULL;
+      }
+      queue->kernelIf->isDataMapped = TRUE;
+   }
+
    return (void *)queue;
 }
 
@@ -939,6 +990,15 @@ VMCI_FreeQueue(void *q,     // IN:
 
    if (queue) {
       uint64 i;
+
+      if (queue->kernelIf->isDataMapped) {
+         ASSERT(queue->kernelIf->va);
+         vunmap(queue->kernelIf->va);
+         queue->kernelIf->va = NULL;
+      }
+
+      ASSERT(NULL == queue->kernelIf->va);
+
       for (i = 0; i < CEILING(size, PAGE_SIZE); i++) {
          __free_page(queue->kernelIf->page[i]);
       }
@@ -1147,9 +1207,15 @@ __VMCIMemcpyToQueue(VMCIQueue *queue,   // OUT:
    while (bytesCopied < size) {
       uint64 pageIndex = (queueOffset + bytesCopied) / PAGE_SIZE;
       size_t pageOffset = (queueOffset + bytesCopied) & (PAGE_SIZE - 1);
-      void *va = kmap(kernelIf->page[pageIndex]);
+      void *va;
       size_t toCopy;
 
+      if (kernelIf->isDataMapped) {
+         va = (void *)((uint8 *)kernelIf->va + (pageIndex * PAGE_SIZE));
+      } else {
+         va = kmap(kernelIf->page[pageIndex]);
+      }
+
       ASSERT(va);
       if (size - bytesCopied > PAGE_SIZE - pageOffset) {
          /* Enough payload to fill up from this page. */
@@ -1165,7 +1231,9 @@ __VMCIMemcpyToQueue(VMCIQueue *queue,   // OUT:
          /* The iovec will track bytesCopied internally. */
          err = memcpy_fromiovec((uint8 *)va + pageOffset, iov, toCopy);
          if (err != 0) {
-            kunmap(kernelIf->page[pageIndex]);
+            if (!kernelIf->isDataMapped) {
+               kunmap(kernelIf->page[pageIndex]);
+            }
             return VMCI_ERROR_INVALID_ARGS;
          }
       } else {
@@ -1173,7 +1241,9 @@ __VMCIMemcpyToQueue(VMCIQueue *queue,   // OUT:
       }
 
       bytesCopied += toCopy;
-      kunmap(kernelIf->page[pageIndex]);
+      if (!kernelIf->isDataMapped) {
+         kunmap(kernelIf->page[pageIndex]);
+      }
    }
 
    return VMCI_SUCCESS;
@@ -1212,9 +1282,15 @@ __VMCIMemcpyFromQueue(void *dest,             // OUT:
    while (bytesCopied < size) {
       uint64 pageIndex = (queueOffset + bytesCopied) / PAGE_SIZE;
       size_t pageOffset = (queueOffset + bytesCopied) & (PAGE_SIZE - 1);
-      void *va = kmap(kernelIf->page[pageIndex]);
+      void *va;
       size_t toCopy;
 
+      if (kernelIf->isDataMapped) {
+         va = (void *)((uint8 *)kernelIf->va + (pageIndex * PAGE_SIZE));
+      } else {
+         va = kmap(kernelIf->page[pageIndex]);
+      }
+
       ASSERT(va);
       if (size - bytesCopied > PAGE_SIZE - pageOffset) {
          /* Enough payload to fill up this page. */
@@ -1230,7 +1306,9 @@ __VMCIMemcpyFromQueue(void *dest,             // OUT:
          /* The iovec will track bytesCopied internally. */
          err = memcpy_toiovec(iov, (uint8 *)va + pageOffset, toCopy);
          if (err != 0) {
-            kunmap(kernelIf->page[pageIndex]);
+            if (!kernelIf->isDataMapped) {
+               kunmap(kernelIf->page[pageIndex]);
+            }
             return VMCI_ERROR_INVALID_ARGS;
          }
       } else {
@@ -1238,7 +1316,9 @@ __VMCIMemcpyFromQueue(void *dest,             // OUT:
       }
 
       bytesCopied += toCopy;
-      kunmap(kernelIf->page[pageIndex]);
+      if (!kernelIf->isDataMapped) {
+         kunmap(kernelIf->page[pageIndex]);
+      }
    }
 
    return VMCI_SUCCESS;
@@ -1267,8 +1347,11 @@ VMCIMemcpyToQueue(VMCIQueue *queue,   // OUT:
                   const void *src,    // IN:
                   size_t srcOffset,   // IN:
                   size_t size,        // IN:
-                  int bufType)        // IN: Unused
+                  int bufType,        // IN: Unused
+                  Bool canBlock)      // IN: Unused
 {
+   ASSERT(canBlock || !queue->kernelIf->host);
+
    return __VMCIMemcpyToQueue(queue, queueOffset,
                               (uint8 *)src + srcOffset, size, FALSE);
 }
@@ -1296,8 +1379,76 @@ VMCIMemcpyFromQueue(void *dest,             // OUT:
                     const VMCIQueue *queue, // IN:
                     uint64 queueOffset,     // IN:
                     size_t size,            // IN:
-                    int bufType)            // IN: Unused
+                    int bufType,            // IN: Unused
+                    Bool canBlock)          // IN: Unused
 {
+   ASSERT(canBlock || !queue->kernelIf->host);
+
+   return __VMCIMemcpyFromQueue((uint8 *)dest + destOffset,
+                                queue, queueOffset, size, FALSE);
+}
+
+
+/*
+ *-----------------------------------------------------------------------------
+ *
+ * VMCIMemcpyToQueueLocal --
+ *
+ *      Copies from a given buffer to a local VMCI queue. On Linux, this is the
+ *      same as a regular copy.
+ *
+ * Results:
+ *      Zero on success, negative error code on failure.
+ *
+ * Side effects:
+ *      None.
+ *
+ *-----------------------------------------------------------------------------
+ */
+
+int
+VMCIMemcpyToQueueLocal(VMCIQueue *queue,     // OUT
+                       uint64 queueOffset,   // IN
+                       const void *src,      // IN
+                       size_t srcOffset,     // IN
+                       size_t size,          // IN
+                       int bufType,          // IN: Unused
+                       Bool canBlock)        // IN: Unused
+{
+   ASSERT(canBlock || !queue->kernelIf->host);
+
+   return __VMCIMemcpyToQueue(queue, queueOffset,
+                              (uint8 *)src + srcOffset, size, FALSE);;
+}
+
+
+/*
+ *-----------------------------------------------------------------------------
+ *
+ * VMCIMemcpyFromQueueLocal --
+ *
+ *      Copies to a given buffer from a VMCI Queue.
+ *
+ * Results:
+ *      Zero on success, negative error code on failure.
+ *
+ * Side effects:
+ *      None.
+ *
+ *-----------------------------------------------------------------------------
+ */
+
+int
+VMCIMemcpyFromQueueLocal(void *dest,             // OUT:
+                         size_t destOffset,      // IN:
+                         const VMCIQueue *queue, // IN:
+                         uint64 queueOffset,     // IN:
+                         size_t size,            // IN:
+                         int bufType,            // IN: Unused
+                         Bool canBlock)          // IN: Unused
+{
+   ASSERT(canBlock || !queue->kernelIf->host);
+
    return __VMCIMemcpyFromQueue((uint8 *)dest + destOffset,
                                 queue, queueOffset, size, FALSE);
 }
@@ -1325,8 +1476,10 @@ VMCIMemcpyToQueueV(VMCIQueue *queue,      // OUT:
                    const void *src,       // IN: iovec
                    size_t srcOffset,      // IN: ignored
                    size_t size,           // IN:
-                   int bufType)           // IN: ignored
+                   int bufType,           // IN: Unused
+                   Bool canBlock)         // IN: Unused
 {
+   ASSERT(canBlock || !queue->kernelIf->host);
 
    /*
     * We ignore srcOffset because src is really a struct iovec * and will
@@ -1358,8 +1511,11 @@ VMCIMemcpyFromQueueV(void *dest,              // OUT: iovec
                      const VMCIQueue *queue,  // IN:
                      uint64 queueOffset,      // IN:
                      size_t size,             // IN:
-                     int bufType)             // IN: ignored
+                     int bufType,             // IN: Unused
+                     Bool canBlock)           // IN: Unused
 {
+   ASSERT(canBlock || !queue->kernelIf->host);
+
    /*
     * We ignore destOffset because dest is really a struct iovec * and will
     * maintain offset internally.
@@ -1426,12 +1582,24 @@ VMCIQueue *
 VMCIHost_AllocQueue(uint64 size) // IN:
 {
    VMCIQueue *queue;
-   const uint queueSize = sizeof *queue + sizeof *(queue->kernelIf);
+   const size_t numPages = CEILING(size, PAGE_SIZE) + 1;
+   const size_t queueSize = sizeof *queue + sizeof *(queue->kernelIf);
+   const size_t queuePageSize = numPages * sizeof *queue->kernelIf->page;
 
-   queue = VMCI_AllocKernelMem(queueSize, VMCI_MEMORY_NORMAL);
+   queue = VMCI_AllocKernelMem(queueSize + queuePageSize, VMCI_MEMORY_NORMAL);
    if (queue) {
       queue->qHeader = NULL;
+      queue->savedHeader = NULL;
       queue->kernelIf = (VMCIQueueKernelIf *)((uint8 *)queue + sizeof *queue);
+      queue->kernelIf->host = TRUE;
+      queue->kernelIf->mutex = NULL;
+      queue->kernelIf->numPages = numPages;
+      queue->kernelIf->headerPage = (struct page **)((uint8*)queue + queueSize);
+      queue->kernelIf->page = &queue->kernelIf->headerPage[1];
+      memset(queue->kernelIf->headerPage, 0,
+             sizeof *queue->kernelIf->headerPage * queue->kernelIf->numPages);
+      queue->kernelIf->va = NULL;
+      queue->kernelIf->isDataMapped = FALSE;
    }
 
    return queue;
@@ -1466,10 +1634,418 @@ VMCIHost_FreeQueue(VMCIQueue *queue,   // IN:
 }
 
 
+/*
+ *-----------------------------------------------------------------------------
+ *
+ * VMCI_InitQueueMutex()
+ *
+ *       Initialize the mutex for the pair of queues.  This mutex is used to
+ *       protect the qHeader and the buffer from changing out from under any
+ *       users of either queue.  Of course, it's only any good if the mutexes
+ *       are actually acquired.  Queue structure must lie on non-paged memory
+ *       or we cannot guarantee access to the mutex.
+ *
+ * Results:
+ *       None.
+ *
+ * Side Effects:
+ *       None.
+ *
+ *----------------------------------------------------------------------------
+ */
+
+void
+VMCI_InitQueueMutex(VMCIQueue *produceQ, // IN/OUT
+                    VMCIQueue *consumeQ) // IN/OUT
+{
+   ASSERT(produceQ);
+   ASSERT(consumeQ);
+   ASSERT(produceQ->kernelIf);
+   ASSERT(consumeQ->kernelIf);
+
+   /*
+    * Only the host queue has shared state - the guest queues do not
+    * need to synchronize access using a queue mutex.
+    */
+
+   if (produceQ->kernelIf->host) {
+      produceQ->kernelIf->mutex = &produceQ->kernelIf->__mutex;
+      consumeQ->kernelIf->mutex = &produceQ->kernelIf->__mutex;
+      sema_init(produceQ->kernelIf->mutex, 1);
+   }
+}
+
+
+/*
+ *-----------------------------------------------------------------------------
+ *
+ * VMCI_CleanupQueueMutex()
+ *
+ *       Cleans up the mutex for the pair of queues.
+ *
+ * Results:
+ *       None.
+ *
+ * Side Effects:
+ *       None.
+ *
+ *----------------------------------------------------------------------------
+ */
+
+void
+VMCI_CleanupQueueMutex(VMCIQueue *produceQ, // IN/OUT
+                       VMCIQueue *consumeQ) // IN/OUT
+{
+   ASSERT(produceQ);
+   ASSERT(consumeQ);
+   ASSERT(produceQ->kernelIf);
+   ASSERT(consumeQ->kernelIf);
+
+   if (produceQ->kernelIf->host) {
+      produceQ->kernelIf->mutex = NULL;
+      consumeQ->kernelIf->mutex = NULL;
+   }
+}
+
+
+/*
+ *-----------------------------------------------------------------------------
+ *
+ * VMCI_AcquireQueueMutex()
+ *
+ *       Acquire the mutex for the queue.  Note that the produceQ and
+ *       the consumeQ share a mutex.  So, only one of the two need to
+ *       be passed in to this routine.  Either will work just fine.
+ *
+ * Results:
+ *       VMCI_SUCCESS always.
+ *
+ * Side Effects:
+ *       May block the caller.
+ *
+ *----------------------------------------------------------------------------
+ */
+
+int
+VMCI_AcquireQueueMutex(VMCIQueue *queue, // IN
+                       Bool canBlock)    // IN: Unused
+{
+   ASSERT(queue);
+   ASSERT(queue->kernelIf);
+
+   if (queue->kernelIf->host) {
+      ASSERT(canBlock);
+      ASSERT(queue->kernelIf->mutex);
+      down(queue->kernelIf->mutex);
+   }
+
+   return VMCI_SUCCESS;
+}
+
+
+/*
+ *-----------------------------------------------------------------------------
+ *
+ * VMCI_ReleaseQueueMutex()
+ *
+ *       Release the mutex for the queue.  Note that the produceQ and
+ *       the consumeQ share a mutex.  So, only one of the two need to
+ *       be passed in to this routine.  Either will work just fine.
+ *
+ * Results:
+ *       None.
+ *
+ * Side Effects:
+ *       May block the caller.
+ *
+ *----------------------------------------------------------------------------
+ */
+
+void
+VMCI_ReleaseQueueMutex(VMCIQueue *queue) // IN
+{
+   ASSERT(queue);
+   ASSERT(queue->kernelIf);
+
+   if (queue->kernelIf->host) {
+      ASSERT(queue->kernelIf->mutex);
+      up(queue->kernelIf->mutex);
+   }
+}
+
+
+/*
+ *-----------------------------------------------------------------------------
+ *
+ * VMCI_LockQueueHeader()
+ *
+ *       Acquire a spinlock guarding the queue header.  Note that the produceQ
+ *       and the consumeQ share the lock mutex.  So, only one of the two need to
+ *       be passed in to this routine.  Either will work just fine.
+ *
+ * Results:
+ *       None.
+ *
+ * Side Effects:
+ *       None.
+ *
+ *----------------------------------------------------------------------------
+ */
+
+void
+VMCI_LockQueueHeader(VMCIQueue *queue) // IN
+{
+   ASSERT(queue);
+   ASSERT(queue->kernelIf);
+   ASSERT(!queue->kernelIf->host);
+
+   /*
+    * We don't support non-blocking on the host right now, so we won't get
+    * here for a host queue.  And there's no lock required on the guest.  So
+    * this is a NOP.
+    */
+}
+
+
+/*
+ *-----------------------------------------------------------------------------
+ *
+ * VMCI_UnlockQueueHeader()
+ *
+ *       Release the spinlock guarding the queue header.
+ *
+ * Results:
+ *       None.
+ *
+ * Side Effects:
+ *       None.
+ *
+ *----------------------------------------------------------------------------
+ */
+
+void
+VMCI_UnlockQueueHeader(VMCIQueue *queue) // IN
+{
+   ASSERT(queue);
+   ASSERT(queue->kernelIf);
+   ASSERT(!queue->kernelIf->host);
+}
+
+
+/*
+ *-----------------------------------------------------------------------------
+ *
+ * VMCIReleasePageStorePages --
+ *
+ *       Helper function to release pages in the PageStoreAttachInfo
+ *       previously obtained using get_user_pages.
+ *
+ * Results:
+ *       None.
+ *
+ * Side Effects:
+ *       None.
+ *
+ *-----------------------------------------------------------------------------
+ */
+
+static void
+VMCIReleasePages(struct page **pages,  // IN
+                 uint64 numPages,      // IN
+                 Bool dirty)           // IN
+{
+   int i;
+
+   for (i = 0; i < numPages; i++) {
+      ASSERT(pages[i]);
+      if (dirty) {
+         set_page_dirty(pages[i]);
+      }
+      page_cache_release(pages[i]);
+      pages[i] = NULL;
+   }
+}
+
+
+/*
+ *-----------------------------------------------------------------------------
+ *
+ * VMCIHost_RegisterUserMemory --
+ *
+ *       Registers the specification of the user pages used for backing a queue
+ *       pair. Enough information to map in pages is stored in the OS specific
+ *       part of the VMCIQueue structure.
+ *
+ * Results:
+ *       VMCI_SUCCESS on sucess, negative error code on failure.
+ *
+ * Side Effects:
+ *       None.
+ *
+ *-----------------------------------------------------------------------------
+ */
+
+int
+VMCIHost_RegisterUserMemory(QueuePairPageStore *pageStore,  // IN
+                            VMCIQueue *produceQ,            // OUT
+                            VMCIQueue *consumeQ)            // OUT
+{
+   VA64 produceUVA;
+   VA64 consumeUVA;
+
+   ASSERT(produceQ->kernelIf->headerPage && consumeQ->kernelIf->headerPage);
+
+   /*
+    * The new style and the old style mapping only differs in that we either
+    * get a single or two UVAs, so we split the single UVA range at the
+    * appropriate spot.
+    */
+
+   produceUVA = pageStore->pages;
+   consumeUVA = pageStore->pages + produceQ->kernelIf->numPages * PAGE_SIZE;
+   return VMCIHost_GetUserMemory(produceUVA, consumeUVA, produceQ, consumeQ);
+}
+
+
+/*
+ *-----------------------------------------------------------------------------
+ *
+ * VMCIHost_UnregisterUserMemory --
+ *
+ *       Releases and removes the references to user pages stored in the attach
+ *       struct.
+ *
+ * Results:
+ *       None
+ *
+ * Side Effects:
+ *       Pages are released from the page cache and may become
+ *       swappable again.
+ *
+ *-----------------------------------------------------------------------------
+ */
+
+void
+VMCIHost_UnregisterUserMemory(VMCIQueue *produceQ,         // IN/OUT
+                              VMCIQueue *consumeQ)         // IN/OUT
+{
+   ASSERT(produceQ->kernelIf);
+   ASSERT(consumeQ->kernelIf);
+   ASSERT(!produceQ->qHeader && !consumeQ->qHeader);
+
+   VMCIReleasePages(produceQ->kernelIf->headerPage, produceQ->kernelIf->numPages, TRUE);
+   memset(produceQ->kernelIf->headerPage, 0,
+          sizeof *produceQ->kernelIf->headerPage * produceQ->kernelIf->numPages);
+   VMCIReleasePages(consumeQ->kernelIf->headerPage, consumeQ->kernelIf->numPages, TRUE);
+   memset(consumeQ->kernelIf->headerPage, 0,
+          sizeof *consumeQ->kernelIf->headerPage * consumeQ->kernelIf->numPages);
+}
+
+
+/*
+ *-----------------------------------------------------------------------------
+ *
+ * VMCIHost_MapQueues --
+ *
+ *       Once VMCIHost_RegisterUserMemory has been performed on a
+ *       queue, the queue pair headers can be mapped into the
+ *       kernel. Once mapped, they must be unmapped with
+ *       VMCIHost_UnmapQueues prior to calling
+ *       VMCIHost_UnregisterUserMemory.
+ *
+ * Results:
+ *       VMCI_SUCCESS if pages are mapped, appropriate error code otherwise.
+ *
+ * Side Effects:
+ *       Pages are pinned.
+ *
+ *-----------------------------------------------------------------------------
+ */
+
+int
+VMCIHost_MapQueues(VMCIQueue *produceQ,  // IN/OUT
+                   VMCIQueue *consumeQ,  // IN/OUT
+                   uint32 flags)         // UNUSED
+{
+   int result;
+
+   if (!produceQ->qHeader || !consumeQ->qHeader) {
+      struct page *headers[2];
+
+      if (produceQ->qHeader != consumeQ->qHeader) {
+         return VMCI_ERROR_QUEUEPAIR_MISMATCH;
+      }
+
+      if (produceQ->kernelIf->headerPage == NULL ||
+          *produceQ->kernelIf->headerPage == NULL) {
+         return VMCI_ERROR_UNAVAILABLE;
+      }
+
+      ASSERT(*produceQ->kernelIf->headerPage && *consumeQ->kernelIf->headerPage);
+
+      headers[0] = *produceQ->kernelIf->headerPage;
+      headers[1] = *consumeQ->kernelIf->headerPage;
+
+      produceQ->qHeader = vmap(headers, 2, VM_MAP, PAGE_KERNEL);
+      if (produceQ->qHeader != NULL) {
+         consumeQ->qHeader =
+            (VMCIQueueHeader *)((uint8 *)produceQ->qHeader + PAGE_SIZE);
+         result = VMCI_SUCCESS;
+      } else {
+         Log("vmap failed\n");
+         result = VMCI_ERROR_NO_MEM;
+     }
+   } else {
+      result = VMCI_SUCCESS;
+   }
+
+   return result;
+}
+
+
+/*
+ *-----------------------------------------------------------------------------
+ *
+ * VMCIHost_UnmapQueues --
+ *
+ *       Unmaps previously mapped queue pair headers from the kernel.
+ *
+ * Results:
+ *       VMCI_SUCCESS always.
+ *
+ * Side Effects:
+ *       Pages are unpinned.
+ *
+ *-----------------------------------------------------------------------------
+ */
+
+int
+VMCIHost_UnmapQueues(VMCIGuestMemID gid,   // IN
+                     VMCIQueue *produceQ,  // IN/OUT
+                     VMCIQueue *consumeQ)  // IN/OUT
+{
+   if (produceQ->qHeader) {
+      ASSERT(consumeQ->qHeader);
+
+      if (produceQ->qHeader < consumeQ->qHeader) {
+         vunmap(produceQ->qHeader);
+      } else {
+         vunmap(consumeQ->qHeader);
+      }
+      produceQ->qHeader = NULL;
+      consumeQ->qHeader = NULL;
+   }
+
+   return VMCI_SUCCESS;
+}
+
+
 /*
  *-----------------------------------------------------------------------------
  *
  * VMCIHost_GetUserMemory --
+ *
+ *
  *       Lock the user pages referenced by the {produce,consume}Buffer
  *       struct into memory and populate the {produce,consume}Pages
  *       arrays in the attach structure with them.
@@ -1484,93 +2060,47 @@ VMCIHost_FreeQueue(VMCIQueue *queue,   // IN:
  */
 
 int
-VMCIHost_GetUserMemory(PageStoreAttachInfo *attach,      // IN/OUT
-                       VMCIQueue *produceQ,              // OUT
-                       VMCIQueue *consumeQ)              // OUT
+VMCIHost_GetUserMemory(VA64 produceUVA,       // IN
+                       VA64 consumeUVA,       // IN
+                       VMCIQueue *produceQ,   // OUT
+                       VMCIQueue *consumeQ)   // OUT
 {
    int retval;
    int err = VMCI_SUCCESS;
 
-
-   attach->producePages =
-      VMCI_AllocKernelMem(attach->numProducePages * sizeof attach->producePages[0],
-                          VMCI_MEMORY_NORMAL);
-   if (attach->producePages == NULL) {
-      return VMCI_ERROR_NO_MEM;
-   }
-   attach->consumePages =
-      VMCI_AllocKernelMem(attach->numConsumePages * sizeof attach->consumePages[0],
-                          VMCI_MEMORY_NORMAL);
-   if (attach->consumePages == NULL) {
-      err = VMCI_ERROR_NO_MEM;
-      goto errorDealloc;
-   }
-
    down_write(&current->mm->mmap_sem);
    retval = get_user_pages(current,
                            current->mm,
-                           (VA)attach->produceBuffer,
-                           attach->numProducePages,
+                           (VA)produceUVA,
+                           produceQ->kernelIf->numPages,
                            1, 0,
-                           attach->producePages,
+                           produceQ->kernelIf->headerPage,
                            NULL);
-   if (retval < attach->numProducePages) {
-      Log("get_user_pages(produce) failed: %d\n", retval);
-      if (retval > 0) {
-         int i;
-         for (i = 0; i < retval; i++) {
-            page_cache_release(attach->producePages[i]);
-         }
-      }
+   if (retval < produceQ->kernelIf->numPages) {
+      Log("get_user_pages(produce) failed (retval=%d)\n", retval);
+      VMCIReleasePages(produceQ->kernelIf->headerPage, retval, FALSE);
       err = VMCI_ERROR_NO_MEM;
       goto out;
    }
 
    retval = get_user_pages(current,
                            current->mm,
-                           (VA)attach->consumeBuffer,
-                           attach->numConsumePages,
+                           (VA)consumeUVA,
+                           consumeQ->kernelIf->numPages,
                            1, 0,
-                           attach->consumePages,
+                           consumeQ->kernelIf->headerPage,
                            NULL);
-   if (retval < attach->numConsumePages) {
-      int i;
-      Log("get_user_pages(consume) failed: %d\n", retval);
-      if (retval > 0) {
-         for (i = 0; i < retval; i++) {
-            page_cache_release(attach->consumePages[i]);
-         }
-      }
-      for (i = 0; i < attach->numProducePages; i++) {
-         page_cache_release(attach->producePages[i]);
-      }
+   if (retval < consumeQ->kernelIf->numPages) {
+      Log("get_user_pages(consume) failed (retval=%d)\n", retval);
+      VMCIReleasePages(consumeQ->kernelIf->headerPage, retval, FALSE);
+      VMCIReleasePages(produceQ->kernelIf->headerPage,
+                       produceQ->kernelIf->numPages, FALSE);
       err = VMCI_ERROR_NO_MEM;
    }
 
 out:
    up_write(&current->mm->mmap_sem);
 
-   if (err == VMCI_SUCCESS) {
-      produceQ->qHeader = kmap(attach->producePages[0]);
-      produceQ->kernelIf->page = &attach->producePages[1];
-      consumeQ->qHeader = kmap(attach->consumePages[0]);
-      consumeQ->kernelIf->page = &attach->consumePages[1];
-   }
-
-errorDealloc:
-   if (err < VMCI_SUCCESS) {
-      if (attach->producePages != NULL) {
-         VMCI_FreeKernelMem(attach->producePages,
-                            attach->numProducePages *
-                            sizeof attach->producePages[0]);
-      }
-      if (attach->consumePages != NULL) {
-         VMCI_FreeKernelMem(attach->consumePages,
-                            attach->numConsumePages *
-                            sizeof attach->consumePages[0]);
-      }
-   }
-
    return err;
 }
 
@@ -1593,38 +2123,12 @@ errorDealloc:
  */
 
 void
-VMCIHost_ReleaseUserMemory(PageStoreAttachInfo *attach,      // IN/OUT
-                           VMCIQueue *produceQ,              // OUT
-                           VMCIQueue *consumeQ)              // OUT
+VMCIHost_ReleaseUserMemory(VMCIQueue *produceQ,  // IN/OUT
+                           VMCIQueue *consumeQ)  // IN/OUT
 {
-   int i;
+   ASSERT(produceQ->kernelIf->headerPage);
 
-   ASSERT(attach->producePages);
-   ASSERT(attach->consumePages);
-
-   kunmap(attach->producePages[0]);
-   kunmap(attach->consumePages[0]);
-
-   for (i = 0; i < attach->numProducePages; i++) {
-      ASSERT(attach->producePages[i]);
-
-      set_page_dirty(attach->producePages[i]);
-      page_cache_release(attach->producePages[i]);
-   }
-
-   for (i = 0; i < attach->numConsumePages; i++) {
-      ASSERT(attach->consumePages[i]);
-
-      set_page_dirty(attach->consumePages[i]);
-      page_cache_release(attach->consumePages[i]);
-   }
-
-   VMCI_FreeKernelMem(attach->producePages,
-                      attach->numProducePages *
-                      sizeof attach->producePages[0]);
-   VMCI_FreeKernelMem(attach->consumePages,
-                      attach->numConsumePages *
-                      sizeof attach->consumePages[0]);
+   VMCIHost_UnregisterUserMemory(produceQ, consumeQ);
 }
 
 
@@ -1646,9 +2150,9 @@ VMCIHost_ReleaseUserMemory(PageStoreAttachInfo *attach,      // IN/OUT
 
 void
 VMCI_ReadPortBytes(VMCIIoHandle handle,  // IN: Unused
-		   VMCIIoPort port,      // IN
-		   uint8 *buffer,        // OUT
-		   size_t bufferLength)  // IN
+                   VMCIIoPort port,      // IN
+                   uint8 *buffer,        // OUT
+                   size_t bufferLength)  // IN
 {
    insb(port, buffer, bufferLength);
 }
