@@ -448,9 +448,14 @@ VSockAddr_SocketContextDgram(uint32 cid,  // IN
    if (cid == VMCI_HYPERVISOR_CONTEXT_ID) {
       /*
        * Registrations of PBRPC Servers do not modify VMX/Hypervisor state and
-       * are allowed.
+       * are allowed.  We also allow messages on the two RPC channels.
+       *
+       * XXX, we should disallow messages to the privileged channel for
+       * sockets that are not owned by admin/root.  See PR 794652.
        */
-      if (rid == VMCI_UNITY_PBRPC_REGISTER) {
+      if (rid == VMCI_UNITY_PBRPC_REGISTER ||
+          rid == VMCI_RPC_PRIVILEGED ||
+          rid == VMCI_RPC_UNPRIVILEGED) {
          return TRUE;
       } else {
          return FALSE;
