@@ -96,50 +96,6 @@ string::string(ConstUnicode s) // IN
 
 
 #ifdef _WIN32
-
-/*
- *-----------------------------------------------------------------------------
- *
- * utf::string::init_bstr_t --
- *
- *      Utility function to construct from a _bstr_t object.
- *      Copies the UTF-16 representation of the _bstr_t.
- *
- * Results:
- *      None.
- *
- * Side effects:
- *      Makes a copy of the _bstr_t data and frees that data when
- *      the utf::string is destroyed.
- *
- * Note:
- *      WIN32 only call
- *
- *-----------------------------------------------------------------------------
- */
-
-void
-string::init_bstr_t(const _bstr_t &s) // IN
-{
-   // If the input is empty, then there's nothing to do.
-   if (s.length() == 0) {
-      return;
-   }
-
-   Unicode utf8 = Unicode_AllocWithUTF16(static_cast<const utf16_t *>(s));
-
-   try {
-      mUstr = utf8;
-      Unicode_Free(utf8);
-   } catch (...) {
-      Unicode_Free(utf8);
-      throw;
-   }
-
-   ASSERT(Validate(mUstr));
-}
-
-
 /*
  *-----------------------------------------------------------------------------
  *
@@ -202,93 +158,23 @@ string::string(const _bstr_t &s) // IN
      mUtf16Cache(NULL),
      mUtf16Length(npos)
 {
-   init_bstr_t(s);
-}
-
-
-/*
- *-----------------------------------------------------------------------------
- *
- * utf::string::string --
- *
- *      Constructor from a uvariant_t object. Copies the UTF-16 representation
- *      of the ubstr_t interface.
- *
- * Results:
- *      None.
- *
- * Side effects:
- *      Makes a copy of the uvariant_t data and frees that data when the
- *      utf::string is destroyed.
- *
- * Note:
- *      WIN32 only call
- *
- *-----------------------------------------------------------------------------
- */
-
-string::string(const uvariant_t &v) // IN
-   : mUstr(),
-     mUtf16Cache(NULL),
-     mUtf16Length(npos)
-{
-   ubstr_t s;
-
-   try {
-      s = v;
-   } catch (...) {
-      Warning("Invalid uvariant_t to ubstr_t conversion.\n");
-      throw;
-   }
-
    // If the input is empty, then there's nothing to do.
    if (s.length() == 0) {
       return;
    }
 
-   mUstr = static_cast<const char *>(s);
-   ASSERT(Validate(mUstr));
-}
-
-
-/*
- *-----------------------------------------------------------------------------
- *
- * utf::string::string --
- *
- *      Constructor from a _variant_t object. Copies the UTF-16 representation
- *      of the _variant_t.
- *
- * Results:
- *      None.
- *
- * Side effects:
- *      Makes a copy of the _variant_t data and frees that data when
- *      the utf::string is destroyed.
- *
- * Note:
- *      WIN32 only call
- *
- *-----------------------------------------------------------------------------
- */
-
-string::string(const _variant_t &v) // IN
-   : mUstr(),
-     mUtf16Cache(NULL),
-     mUtf16Length(npos)
-{
-   _bstr_t s;
+   Unicode utf8 = Unicode_AllocWithUTF16(static_cast<const utf16_t *>(s));
 
    try {
-      s = v;
+      mUstr = utf8;
+      Unicode_Free(utf8);
    } catch (...) {
-      Warning("Invalid _variant_t to _bstr_t conversion.\n");
+      Unicode_Free(utf8);
       throw;
    }
 
-   init_bstr_t(s);
+   ASSERT(Validate(mUstr));
 }
-
 #endif
 
 
@@ -741,6 +627,30 @@ string::empty()
 }
 
 
+/*
+ *-----------------------------------------------------------------------------
+ *
+ * utf::string::isASCII --
+ *
+ *      Test if every character in the string is in the ASCII range.
+ *
+ * Results:
+ *      true if all ASCII, otherwise false.
+ *
+ * Side effects:
+ *      None
+ *
+ *-----------------------------------------------------------------------------
+ */
+
+bool
+string::isASCII()
+   const
+{
+   return mUstr.is_ascii();
+}
+
+
 /*
  *-----------------------------------------------------------------------------
  *
@@ -1840,11 +1750,15 @@ string::endsWith(const string &s, // IN
  *
  *      Return a vector of utf::strings.  The vector contains the elements of
  *      the string split by the passed in separator. Empty tokens are not
- *      skipped.
+ *      skipped. If maxStrings is zero, any number of strings will be returned,
+ *      otherwise parsing stops after maxStrings - 1 matches of the separator.
+ *      In that case, the last string returned includes the rest of the
+ *      original string.
  *
  *      "1,2,3".split(",") -> ["1", "2", "3"]
  *      "1,,".split(",") -> ["1", "", ""]
  *      "1".split(",") -> ["1"]
+ *      "1,2,3".split(",", 2) -> ["1", "2,3"]
  *
  *      XXX If this is to be used for things like command line parsing, support
  *      for quoted strings needs to be added.
@@ -1859,18 +1773,21 @@ string::endsWith(const string &s, // IN
  */
 
 std::vector<string>
-string::split(const string &sep) // IN
+string::split(const string &sep, // IN
+              size_t maxStrings) // IN/OPT
    const
 {
    std::vector<string> splitStrings;
    size_type sIndex = 0;
    size_type sepLen = sep.length();
+   size_t count = 0;
 
    ASSERT(sepLen > 0);
 
    while (true) {
       size_type index = find(sep, sIndex);
-      if (index == npos) {
+      count++;
+      if (count == maxStrings || index == npos) {
          splitStrings.push_back(substr(sIndex));
          break;
       }
